@@ -2,10 +2,17 @@
 
 set -e
 
-if [ -d .git ]; then
+if [ "$1" = "--preload" ]; then
+    shift
+    . "$1"
+    shift
+fi
+
+if [ -e .git ]; then
     : ${BELENIOS_CONFIG:=demo/ocsigenserver.conf.in}
     : ${BELENIOS_VARDIR:=_run}
     : ${BELENIOS_RUNDIR:=/tmp/belenios}
+    : ${BELENIOS_BINDIR:=_run/usr/bin}
     : ${BELENIOS_LIBDIR:=_run/usr/lib}
     : ${BELENIOS_SHAREDIR:=_run/usr/share/belenios-server}
 fi
@@ -18,8 +25,14 @@ check_nonempty_var () {
 }
 
 check_nonempty_var BELENIOS_CONFIG
+
+if [ -f ${BELENIOS_CONFIG}.preload ]; then
+   . ${BELENIOS_CONFIG}.preload
+fi
+
 check_nonempty_var BELENIOS_VARDIR
 check_nonempty_var BELENIOS_RUNDIR
+check_nonempty_var BELENIOS_BINDIR
 check_nonempty_var BELENIOS_LIBDIR
 check_nonempty_var BELENIOS_SHAREDIR
 
@@ -28,9 +41,14 @@ mkdir -p \
       $BELENIOS_VARDIR/log \
       $BELENIOS_VARDIR/lib \
       $BELENIOS_VARDIR/upload \
-      $BELENIOS_VARDIR/spool \
+      $BELENIOS_VARDIR/accounts \
       $BELENIOS_RUNDIR
 
+if ! [ -d $BELENIOS_VARDIR/spool ]; then
+    mkdir $BELENIOS_VARDIR/spool
+    echo 1 > $BELENIOS_VARDIR/spool/version
+fi
+
 touch $BELENIOS_VARDIR/password_db.csv
 
 sed \
@@ -40,10 +58,8 @@ sed \
     -e "s@_SHAREDIR_@$BELENIOS_SHAREDIR@g" \
     $BELENIOS_CONFIG > $BELENIOS_VARDIR/etc/ocsigenserver.conf
 
-OCSIGENSERVER=ocsigenserver
+PATH=$BELENIOS_BINDIR:$PATH:/usr/sbin
 
-if command -v ocsigenserver.opt > /dev/null; then
-  OCSIGENSERVER=ocsigenserver.opt
-fi
+OCSIGENSERVER=belenios-server
 
 exec $OCSIGENSERVER -c $BELENIOS_VARDIR/etc/ocsigenserver.conf "$@"
