@@ -23,6 +23,8 @@ open Belenios_platform
 open Platform
 open Signatures_core
 
+let ( >> ) f g x = g (f x)
+let ( ^^^ ) a b = a ^ " — " ^ b
 let ( let@ ) f x = f x
 let ( let& ) = Option.bind
 let ( // ) = Filename.concat
@@ -40,9 +42,17 @@ module Array = Common_types.Array
 module Shape = Common_types.Shape
 
 let sha256_hex = Crypto_primitives.sha256_hex
-let sha256_b64 x = Hash.hash_string x |> Hash.to_b64
+let sha256_b64 = Hash.hash_string >> Hash.to_b64
 let b58_digits = Common_types.b58_digits
 
+let encode_data_uri ?charset ~mime_type x =
+  let charset =
+    match charset with None -> "" | Some x -> Printf.sprintf "charset=%s;" x
+  in
+  match Base64.encode x with
+  | Ok x -> Printf.sprintf "data:%s;%sbase64,%s" mime_type charset x
+  | Error (`Msg msg) -> failwith msg
+
 module String = struct
   include String
 
@@ -99,7 +109,7 @@ struct
   let to_int x = Z.to_int x
   let zero = of_int 0
   let one = of_int 1
-  let to_Z x = x
+  let to_Z = Fun.id
   let reduce x = Z.erem x q
   let reduce_hex x = Z.(erem (of_hex x) q)
 
@@ -276,6 +286,15 @@ let rev_split_lines str =
 
 let split_lines str = List.rev @@ rev_split_lines str
 
+let join_lines xs =
+  let b = Buffer.create 1024 in
+  List.iter
+    (fun x ->
+      Buffer.add_string b x;
+      Buffer.add_char b '\n')
+    xs;
+  Buffer.contents b
+
 let parse_public_credential of_string x =
   let open Serializable_core_t in
   match String.split_on_char ',' x with
@@ -325,8 +344,8 @@ let format_public_credential to_string x =
   | Some w ->
       Printf.sprintf "%s,%s" (to_string x.credential) (Weight.to_string w)
 
-let strip_public_credential x =
-  x |> parse_public_credential Fun.id |> format_public_credential Fun.id
+let strip_public_credential =
+  parse_public_credential Fun.id >> format_public_credential Fun.id
 
 let extract_salt x = (parse_public_credential Fun.id x).salt
 let re_exec_opt ~rex x = try Some (Re.Pcre.exec ~rex x) with Not_found -> None
@@ -371,6 +390,22 @@ let map_and_concat_with_commas f xs =
   let size = Buffer.length res - 1 in
   if size > 0 then Buffer.sub res 0 size else ""
 
+let remove_special_characters x =
+  let n = String.length x in
+  let b = Buffer.create n in
+  let convert = function
+    | ('0' .. '9' | 'A' .. 'Z' | 'a' .. 'z' | '-') as c -> c
+    | _ -> '_'
+  in
+  let rec loop last i =
+    if i < n then (
+      let c = convert x.[i] in
+      if c <> '_' || last <> '_' then Buffer.add_char b c;
+      loop c (i + 1))
+    else Buffer.contents b
+  in
+  loop '\000' 0
+
 module Voter = struct
   type t = [ `Plain | `Json ] * Serializable_core_t.voter
 
