@@ -62,8 +62,8 @@ module Parse (R : RAW_ELECTION) () = struct
   let read_question a b = Question.of_concrete (read_question a b)
   let write_question b x = write_question b (Question.to_concrete x)
 
-  let erase_question x =
-    x |> Question.to_concrete |> erase_question |> Question.of_concrete
+  let erase_question =
+    Question.to_concrete >> erase_question >> Question.of_concrete
 
   let j = params_of_string Yojson.Safe.read_json R.raw_election
 
@@ -123,7 +123,7 @@ struct
   let random () = Zq.random (M.get_rng ())
   let ( / ) x y = x *~ invert y
 
-  type plaintext = int array array
+  type plaintext = int Shape.t array
   type nonrec ballot = (element, scalar) ballot
   type weighted_ballot = Weight.t * ballot
 
@@ -212,7 +212,7 @@ struct
     match
       ballot_of_string (sread G.of_string) (sread G.Zq.of_string) rawballot
     with
-    | exception e -> Error (`SerializationError e)
+    | exception e -> Error (`SerializationError (Printexc.to_string e))
     | ballot ->
         if
           string_of_ballot (swrite G.to_string) (swrite G.Zq.to_string) ballot
