@@ -19,41 +19,58 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
+open Lwt.Syntax
 open Belenios
 open Common
 
 let parse_config x =
   sealing_config_of_string x
   |> List.map (fun (r, i) ->
-         (Str.regexp r, List.fold_left (Fun.flip SSet.add) SSet.empty i))
+         (Re.Str.regexp r, List.fold_left (Fun.flip SSet.add) SSet.empty i))
 
 let find_ign cfg path =
   List.filter_map
-    (fun (r, i) -> if Str.string_match r path 0 then Some i else None)
+    (fun (r, i) -> if Re.Str.string_match r path 0 then Some i else None)
     cfg
   |> List.fold_left SSet.union SSet.empty
 
-let rec measure full cfg ign path =
+let rec measure pool full cfg ign path =
   let make name x = if SSet.mem name ign then None else Some x in
-  let st = Unix.LargeFile.lstat path in
-  let st_contents =
-    if SSet.mem "contents" ign then None
+  let* st = Lwt_unix.LargeFile.lstat path in
+  let* st_contents =
+    if SSet.mem "contents" ign then Lwt.return_none
     else
       match st.st_kind with
-      | S_CHR | S_BLK | S_FIFO | S_SOCK -> None
-      | S_REG -> Some (`REG (Hash.hash_string (string_of_file path)))
-      | S_LNK -> Some (`LNK (Unix.readlink path))
+      | S_CHR | S_BLK | S_FIFO | S_SOCK -> Lwt.return_none
+      | S_REG ->
+          let* contents =
+            let@ () = Lwt_pool.use pool in
+            let@ ic = Lwt_io.with_file ~mode:Input path in
+            Lwt_io.read ic
+          in
+          Lwt.return_some @@ `REG (Hash.hash_string contents)
+      | S_LNK ->
+          let* contents = Lwt_unix.readlink path in
+          Lwt.return_some @@ `LNK contents
       | S_DIR ->
+          let* files =
+            let@ () = Lwt_pool.use pool in
+            Lwt_unix.files_of_directory path |> Lwt_stream.to_list
+          in
           let files =
-            Sys.readdir path |> Array.to_list
+            files
             |> List.filter (fun x -> x <> "." && x <> "..")
             |> List.sort String.compare
-            |> List.map (fun x ->
+          in
+          let* files =
+            files
+            |> Lwt_list.map_p (fun x ->
                    let path = path // x in
                    let ign = find_ign cfg path in
-                   ((if full then path else x), measure full cfg ign path))
+                   let* r = measure pool full cfg ign path in
+                   Lwt.return ((if full then path else x), r))
           in
-          Some (`DIR files)
+          Lwt.return_some @@ `DIR files
   in
   let kind =
     match st.st_kind with
@@ -80,15 +97,22 @@ let rec measure full cfg ign path =
     st_ctime = make "ctime" st.st_ctime;
     st_contents;
   }
+  |> Lwt.return
 
-let main full cfg path =
-  let cfg =
-    match cfg with None -> [] | Some x -> parse_config (string_of_file x)
+let main full cfg path jobs =
+  let@ () = wrap_main in
+  let* cfg =
+    match cfg with
+    | None -> Lwt.return_nil
+    | Some x ->
+        let* x = string_of_file x in
+        Lwt.return @@ parse_config x
   in
   let ign = find_ign cfg path in
-  let x = measure full cfg ign path in
-  print_endline (string_of_stats x);
-  `Ok ()
+  let pool = Lwt_pool.create jobs (fun () -> Lwt.return_unit) in
+  let* x = measure pool full cfg ign path in
+  let* () = Lwt_io.printl (string_of_stats x) in
+  Lwt.return_unit
 
 open Cmdliner
 
@@ -113,6 +137,11 @@ let path_t =
   let the_info = Arg.info [ "path" ] ~docv:"PATH" ~doc in
   Arg.(value & opt file Filename.current_dir_name the_info)
 
+let jobs_t =
+  let doc = "Use up to $(docv) parallel I/O jobs" in
+  let the_info = Arg.info [ "jobs" ] ~docv:"JOBS" ~doc in
+  Arg.(value & opt int 1 the_info)
+
 let cmd =
   let doc = "measure a directory" in
   let man =
@@ -128,4 +157,4 @@ let cmd =
   in
   Cmd.v
     (Cmd.info "measure" ~doc ~man)
-    Term.(ret (const main $ full_t $ cfg_t $ path_t))
+    Term.(ret (const main $ full_t $ cfg_t $ path_t $ jobs_t))
