@@ -19,6 +19,7 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
+open Lwt.Syntax
 open Belenios
 
 let block_size = Archive.block_size
@@ -33,46 +34,71 @@ type index = {
   header : archive_header;
 }
 
-module DirectMonad = struct
-  type 'a t = 'a
+module LwtMonad = struct
+  type 'a t = 'a Lwt.t
 
-  let return x = x
-  let bind x f = f x
-  let fail x = raise x
-  let yield () = ()
+  let return = Lwt.return
+  let bind = Lwt.bind
+  let fail = Lwt.fail
+  let yield = Lwt.pause
 end
 
+type file = { mutable pos : int64; fd : Lwt_unix.file_descr }
+
 module IoReader = struct
-  include DirectMonad
+  include LwtMonad
 
-  type file = in_channel
+  type nonrec file = file
 
-  let get_pos = LargeFile.pos_in
-  let set_pos = LargeFile.seek_in
-  let read_block ic buffer = really_input ic buffer 0 block_size
+  let get_pos f = Lwt.return f.pos
+
+  let set_pos f x =
+    let* _ = Lwt_unix.LargeFile.lseek f.fd x SEEK_SET in
+    f.pos <- x;
+    Lwt.return_unit
+
+  let read_block f buffer =
+    let rec loop i =
+      if i < block_size then (
+        let* n = Lwt_unix.read f.fd buffer i (block_size - i) in
+        f.pos <- Int64.add f.pos (Int64.of_int n);
+        if n = 0 then raise End_of_file else loop (i + n))
+      else Lwt.return_unit
+    in
+    loop 0
 end
 
 module Reader = Archive.MakeReader (IoReader)
 
 module IoWriter = struct
-  include DirectMonad
+  include LwtMonad
 
-  type file = out_channel
+  type nonrec file = file
 
-  let get_pos = LargeFile.pos_out
-  let write_block oc buffer = output_bytes oc buffer
+  let get_pos f = Lwt.return f.pos
+
+  let write_block f buffer =
+    let rec loop i =
+      if i < block_size then (
+        let* n = Lwt_unix.write f.fd buffer i (block_size - i) in
+        f.pos <- Int64.add f.pos (Int64.of_int n);
+        loop (i + n))
+      else Lwt.return_unit
+    in
+    loop 0
 end
 
 module Writer = Archive.MakeWriter (IoWriter)
 
 let build_index filename =
   let r = Hashtbl.create 1000 in
-  let ic = open_in filename in
-  let header = Reader.read_header ic in
+  let* fd = Lwt_unix.openfile filename [ O_RDONLY ] 0o600 in
+  let ic = { pos = 0L; fd } in
+  let* header = Reader.read_header ic in
   let rec loop last accu lines =
-    match Reader.read_record ic with
-    | exception End_of_file -> (r, last, accu, lines, header)
-    | record ->
+    Lwt.try_bind
+      (fun () -> Reader.read_record ic)
+      (fun record ->
         let last, accu =
           match record.typ with
           | Data -> (last, accu)
@@ -80,55 +106,74 @@ let build_index filename =
               (Some event, Events.update_roots record.hash event accu)
         in
         Hashtbl.add r record.hash record.location;
-        loop last accu ((record.typ, record.hash) :: lines)
+        loop last accu ((record.typ, record.hash) :: lines))
+      (function
+        | End_of_file -> Lwt.return (r, last, accu, lines, header)
+        | e -> Lwt.reraise e)
   in
-  Fun.protect
-    ~finally:(fun () -> close_in ic)
+  Lwt.finalize
     (fun () -> loop None Events.empty_roots [])
+    (fun () -> Lwt_unix.close fd)
 
 let get_index ~file =
-  let map, last_event, roots, lines, header = build_index file in
+  let* map, last_event, roots, lines, header = build_index file in
   let timestamp = Archive.get_timestamp header in
-  { map; roots; last_event; file; lines; timestamp; header }
+  { map; roots; last_event; file; lines; timestamp; header } |> Lwt.return
+
+let really_input_string ic n =
+  let buffer = Bytes.create n in
+  let rec loop i =
+    if i < n then
+      let* j = Lwt_unix.read ic buffer i (n - i) in
+      loop (i + j)
+    else Lwt.return @@ Bytes.to_string buffer
+  in
+  loop 0
 
 let gethash ~index ~filename x =
   match Hashtbl.find_opt index x with
-  | None -> None
+  | None -> Lwt.return_none
   | Some i ->
-      let ic = open_in filename in
-      Fun.protect
-        ~finally:(fun () -> close_in ic)
+      let* ic = Lwt_unix.openfile filename [ O_RDONLY ] 0o600 in
+      Lwt.finalize
         (fun () ->
-          LargeFile.seek_in ic i.location_offset;
+          let* _ = Lwt_unix.LargeFile.lseek ic i.location_offset SEEK_SET in
           assert (i.location_length <= Int64.of_int Sys.max_string_length);
-          Some (really_input_string ic (Int64.to_int i.location_length)))
+          let* contents =
+            really_input_string ic (Int64.to_int i.location_length)
+          in
+          Lwt.return_some contents)
+        (fun () -> Lwt_unix.close ic)
 
 let get_data i x = gethash ~index:i.map ~filename:i.file x
 
 let get_event i x =
-  gethash ~index:i.map ~filename:i.file x |> Option.map event_of_string
+  let* x = gethash ~index:i.map ~filename:i.file x in
+  Lwt.return @@ Option.map event_of_string x
 
 let get_roots i = i.roots
 
 let fold_on_event_payload_hashes index typ last_event f accu =
   let rec loop e accu =
-    match get_event index e with
+    let* x = get_event index e in
+    match x with
     | None -> assert false
     | Some e ->
         if e.event_typ = typ then
           match (e.event_payload, e.event_parent) with
-          | Some payload, Some parent -> loop parent (f payload accu)
+          | Some payload, Some parent ->
+              let* x = f payload accu in
+              loop parent x
           | _ -> assert false
-        else accu
+        else Lwt.return accu
   in
   loop last_event accu
 
 let fold_on_event_payloads index typ last_event f accu =
   fold_on_event_payload_hashes index typ last_event
     (fun payload accu ->
-      match get_data index payload with
-      | None -> assert false
-      | Some x -> f x accu)
+      let* x = get_data index payload in
+      match x with None -> assert false | Some x -> f x accu)
     accu
 
 let fsck index =
@@ -136,36 +181,47 @@ let fsck index =
     match index.last_event with None -> failwith "no events" | Some x -> x
   in
   let module IoComparer = struct
-    include DirectMonad
+    include LwtMonad
 
-    type file = in_channel
+    type nonrec file = file
 
-    let get_pos = LargeFile.pos_in
+    let get_pos f = Lwt.return f.pos
     let buffer = Bytes.create block_size
 
-    let write_block ic expected =
+    let write_block f expected =
       try
-        let () = really_input ic buffer 0 block_size in
+        let rec loop i =
+          if i < block_size then (
+            let* n = Lwt_unix.read f.fd buffer i (block_size - i) in
+            f.pos <- Int64.add f.pos (Int64.of_int n);
+            loop (i + n))
+          else Lwt.return_unit
+        in
+        let* () = loop 0 in
         if expected <> buffer then
           failwith "generated archive is not identical to original one"
+        else Lwt.return_unit
       with End_of_file ->
-        failwith "generate archive is longer than original one"
+        failwith "generated archive is longer than original one"
   end in
   let module Comparer = Archive.MakeWriter (IoComparer) in
   let module IoArchiver = struct
-    include DirectMonad
+    include LwtMonad
 
     let get_hash hash = gethash ~index:index.map ~filename:index.file hash
   end in
   let module Archiver = Archive.MakeArchiver (IoArchiver) (Comparer) in
-  let ic = open_in_bin index.file in
-  let length = LargeFile.in_channel_length ic in
-  Fun.protect
-    ~finally:(fun () -> close_in ic)
+  let* fd = Lwt_unix.openfile index.file [ O_RDONLY ] 0o600 in
+  let* length = Lwt_unix.LargeFile.lseek fd 0L SEEK_END in
+  let* _ = Lwt_unix.LargeFile.lseek fd 0L SEEK_SET in
+  let ic = { pos = 0L; fd } in
+  Lwt.finalize
     (fun () ->
-      let () = Archiver.write_archive ic index.header last_event in
-      if LargeFile.pos_in ic <> length then
-        failwith "generated archive is shorter than original one")
+      let* () = Archiver.write_archive ic index.header last_event in
+      if ic.pos <> length then
+        failwith "generated archive is shorter than original one"
+      else Lwt.return_unit)
+    (fun () -> Lwt_unix.close fd)
 
 let starts_with ~(prefix : index) (index : index) =
   let rec loop x y =
@@ -177,25 +233,25 @@ let starts_with ~(prefix : index) (index : index) =
   loop (List.rev prefix.lines) (List.rev index.lines)
 
 let write_header filename header =
-  let oc =
-    open_out_gen
-      [ Open_wronly; Open_append; Open_creat; Open_binary ]
-      0o644 filename
-  in
-  Fun.protect
-    ~finally:(fun () -> close_out oc)
+  let* fd = Lwt_unix.openfile filename [ O_WRONLY; O_APPEND; O_CREAT ] 0o644 in
+  let* pos = Lwt_unix.LargeFile.lseek fd 0L SEEK_CUR in
+  let oc = { pos; fd } in
+  Lwt.finalize
     (fun () -> Writer.write_header oc header)
+    (fun () -> Lwt_unix.close fd)
 
 let raw_append ~filename ~timestamp xs =
-  let oc =
-    open_out_gen [ Open_wronly; Open_append; Open_binary ] 0o644 filename
-  in
-  Fun.protect
-    ~finally:(fun () -> close_out oc)
+  let* fd = Lwt_unix.openfile filename [ O_WRONLY; O_APPEND ] 0o644 in
+  let* pos = Lwt_unix.LargeFile.lseek fd 0L SEEK_CUR in
+  let oc = { pos; fd } in
+  Lwt.finalize
     (fun () ->
-      List.fold_left
-        (fun accu (typ, x) -> Writer.write_record oc ~timestamp typ x :: accu)
+      Lwt_list.fold_left_s
+        (fun accu (typ, x) ->
+          let* y = Writer.write_record oc ~timestamp typ x in
+          Lwt.return @@ (y :: accu))
         [] xs)
+    (fun () -> Lwt_unix.close fd)
 
 type append_operation = Data of string | Event of event_type * hash option
 
@@ -230,16 +286,18 @@ let append index ops =
       ops
   in
   let items = List.rev items in
-  let records =
+  let* records =
     raw_append ~filename:index.file ~timestamp:index.timestamp items
   in
   List.iter (fun r -> Hashtbl.add index.map r.Archive.hash r.location) records;
   index.roots <- roots;
   index.last_event <- last_event;
-  index.lines <- lines
+  index.lines <- lines;
+  Lwt.return_unit
 
 let init ~file ~election ~trustees ~public_creds =
-  if Sys.file_exists file then Printf.ksprintf failwith "%s already exists" file;
+  let* b = Lwt_unix.file_exists file in
+  if b then Printf.ksprintf failwith "%s already exists" file;
   let header = Archive.new_header () in
   let index =
     {
@@ -252,18 +310,20 @@ let init ~file ~election ~trustees ~public_creds =
       header;
     }
   in
-  write_header file header;
+  let* () = write_header file header in
   let setup_election = Hash.hash_string election in
   let setup_trustees = Hash.hash_string trustees in
   let setup_credentials = Hash.hash_string public_creds in
   let setup_data = { setup_election; setup_trustees; setup_credentials } in
   let setup_data_s = string_of_setup_data setup_data in
-  append index
-    [
-      Data election;
-      Data trustees;
-      Data public_creds;
-      Data setup_data_s;
-      Event (`Setup, Some (Hash.hash_string setup_data_s));
-    ];
-  index
+  let* () =
+    append index
+      [
+        Data election;
+        Data trustees;
+        Data public_creds;
+        Data setup_data_s;
+        Event (`Setup, Some (Hash.hash_string setup_data_s));
+      ]
+  in
+  Lwt.return index
