@@ -19,50 +19,58 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
+open Lwt.Syntax
 open Belenios
 open Common
 
 let get_last_event dir =
   let rec loop ((accu_event, accu_files) as accu) = function
-    | [] -> accu
+    | [] -> Lwt.return accu
     | file :: files ->
         let accu_files = SSet.add file accu_files in
-        let accu_event =
+        let* accu_event =
           if Filename.check_suffix file ".event.json" then
-            let event_s = string_of_file (dir // file) in
+            let* event_s = string_of_file (dir // file) in
             let event = event_of_string event_s in
             match accu_event with
-            | Some old when old.event_height > event.event_height -> accu_event
-            | _ -> Some event
-          else accu_event
+            | Some old when old.event_height > event.event_height ->
+                Lwt.return accu_event
+            | _ -> Lwt.return_some event
+          else Lwt.return accu_event
         in
         loop (accu_event, accu_files) files
   in
-  loop (None, SSet.empty) (Sys.readdir dir |> Array.to_list)
+  let* files = Lwt_unix.files_of_directory dir |> Lwt_stream.to_list in
+  loop (None, SSet.empty) files
 
 let mkarchive dir =
-  let header = string_of_file (dir // "BELENIOS") |> archive_header_of_string in
-  let last_event, files = get_last_event dir in
+  let* header =
+    let* x = string_of_file (dir // "BELENIOS") in
+    Lwt.return @@ archive_header_of_string x
+  in
+  let* last_event, files = get_last_event dir in
   let last_event =
     match last_event with Some x -> x | None -> failwith "no events found"
   in
   let module IoArchiver = struct
-    include Tool_events.DirectMonad
+    include Tool_events.LwtMonad
 
     let get_hash hash =
       let hash_s = Hash.to_hex hash in
       let event_filename = hash_s ^ ".event.json" in
       let data_filename = hash_s ^ ".data.json" in
       if SSet.mem event_filename files then
-        Some (string_of_file (dir // event_filename))
+        let* contents = string_of_file (dir // event_filename) in
+        Lwt.return_some contents
       else if SSet.mem data_filename files then
-        Some (string_of_file (dir // data_filename))
-      else None
+        let* contents = string_of_file (dir // data_filename) in
+        Lwt.return_some contents
+      else Lwt.return_none
   end in
   let module Archiver = Archive.MakeArchiver (IoArchiver) (Tool_events.Writer)
   in
-  set_binary_mode_out stdout true;
-  Archiver.write_archive stdout header last_event
+  let oc : Tool_events.file = { pos = 0L; fd = Lwt_unix.stdout } in
+  Archiver.write_archive oc header last_event
 
 let main dir =
   let@ () = wrap_main in
