@@ -31,7 +31,7 @@ open Common
 
 let send_draft_request req =
   let uuid = get_current_uuid () in
-  let* x = post_with_token (string_of_draft_request req) "drafts/%s" uuid in
+  let* x = Api.(post (draft uuid) !user req) in
   if x.code <> 200 then
     alert ("Draft request failed with error code " ^ string_of_int x.code);
   Cache.invalidate Cache.status;
@@ -47,33 +47,29 @@ let step = ref 0
 (* Forward decl of update functions *)
 let update_main_zone = ref (fun _ -> Lwt.return_unit)
 
-let cast_bt_trustee x =
-  x
-  |> string_of_trustee
-       (write_trustee_public_key Yojson.Safe.write_json Yojson.Safe.write_json)
-  |> trustee_of_string Yojson.Safe.read_json
+let cast_bt_trustee =
+  string_of_trustee
+    (write_trustee_public_key Yojson.Safe.write_json Yojson.Safe.write_json)
+  >> trustee_of_string Yojson.Safe.read_json
 
-let cast_tt_trustee x =
-  x
-  |> string_of_trustee (write_cert Yojson.Safe.write_json)
-  |> trustee_of_string Yojson.Safe.read_json
+let cast_tt_trustee =
+  string_of_trustee (write_cert Yojson.Safe.write_json)
+  >> trustee_of_string Yojson.Safe.read_json
 
 let get_trustees () =
   let uuid = get_current_uuid () in
   let* status = Cache.get_until_success Cache.status in
   step := status.trustees_setup_step;
-  let* x =
-    get
-      (draft_trustees_of_string Yojson.Safe.read_json Yojson.Safe.read_json)
-      "drafts/%s/trustees" uuid
-  in
+  let* x = Api.(get (draft_trustees uuid) !user) in
   ifmatch_tt := get_ifmatch x;
   match x with
   | Error e ->
       alert (string_of_error e);
       Lwt.return_unit
   | Ok (tt, _) -> (
-      match tt with
+      match
+        draft_trustees_of_string Yojson.Safe.read_json Yojson.Safe.read_json tt
+      with
       | `Basic x ->
           all_trustee := List.map cast_bt_trustee x.bt_trustees;
           mode := `Basic;
@@ -87,16 +83,11 @@ let recompute_main_zone_1 () =
   let open (val !Belenios_js.I18n.gettext) in
   let uuid = get_current_uuid () in
   let erase_trustee_elt t =
-    let encoded_trustee =
-      t |> Js.string |> Js.encodeURIComponent |> Js.to_string
-    in
     let elt = div ~a:[ a_class [ "del_sym" ] ] [] in
     let r = Tyxml_js.To_dom.of_div elt in
     r##.onclick :=
       lwt_handler (fun () ->
-          let* x =
-            delete_with_token "drafts/%s/trustees/%s" uuid encoded_trustee
-          in
+          let* x = Api.(delete (draft_trustee uuid t) !user) in
           match x.code with
           | 200 -> !update_main_zone ()
           | code ->
@@ -164,11 +155,8 @@ let recompute_main_zone_1 () =
             }
           in
           let r = `Add t in
-          let* x =
-            post_with_token ?ifmatch:!ifmatch_tt
-              (string_of_trustees_request r)
-              "drafts/%s/trustees" uuid
-          in
+          let ifmatch = !ifmatch_tt in
+          let* x = Api.(post ?ifmatch (draft_trustees uuid) !user r) in
           let&&* d = document##getElementById (Js.string "popup") in
           d##.style##.display := Js.string "none";
           match x.code with
@@ -228,12 +216,9 @@ let recompute_main_zone_1 () =
           in
           if ok then (
             let with_thr = not with_thr in
-            let mm =
-              string_of_trustees_request
-                (if with_thr then `SetThreshold 0 else `SetBasic)
-            in
+            let mm = if with_thr then `SetThreshold 0 else `SetBasic in
             let ifmatch = !ifmatch_tt in
-            let* x = post_with_token ?ifmatch mm "drafts/%s/trustees" uuid in
+            let* x = Api.(post ?ifmatch (draft_trustees uuid) !user mm) in
             match x.code with
             | 200 -> !update_main_zone ()
             | _ ->
@@ -261,9 +246,9 @@ let recompute_main_zone_1 () =
       r##.onchange :=
         lwt_handler (fun _ ->
             let vv = int_of_string (Js.to_string r##.value) in
-            let mm = string_of_trustees_request (`SetThreshold vv) in
+            let mm = `SetThreshold vv in
             let ifmatch = !ifmatch_tt in
-            let* x = post_with_token ?ifmatch mm "drafts/%s/trustees" uuid in
+            let* x = Api.(post ?ifmatch (draft_trustees uuid) !user mm) in
             match x.code with
             | 200 -> !update_main_zone ()
             | _ ->
@@ -281,20 +266,36 @@ let recompute_main_zone_1 () =
     else div [ inp; lab ]
   in
   let proc_but =
-    button (s_ "Proceed to key generation") (fun () ->
-        if match !mode with `Basic -> false | `Threshold i -> i = 0 then (
-          alert "Threshold has not been set";
-          Lwt.return_unit)
-        else
-          (* TODO: are there more consistency checks to do, here? *)
-          let confirm =
-            confirm
-            @@ s_ "Proceed to next step? This will freeze the list of trustees."
-          in
-          if confirm then
-            let* () = send_draft_request @@ `SetTrusteesSetupStep 2 in
-            !update_main_zone ()
-          else Lwt.return_unit)
+    let@ () = button (s_ "Proceed to key generation") in
+    match !mode with
+    | `Threshold 0 ->
+        alert "Threshold has not been set";
+        Lwt.return_unit
+    | _ ->
+        (* TODO: are there more consistency checks to do, here? *)
+        let confirm =
+          confirm
+          @@ s_ "Proceed to next step? This will freeze the list of trustees."
+        in
+        if confirm then
+          let* () = send_draft_request @@ `SetTrusteesSetupStep 2 in
+          !update_main_zone ()
+        else Lwt.return_unit
+  in
+  let import_but =
+    let@ () = button @@ s_ "Import trustees from another election" in
+    let@ from_uuid = popup_choose_elec uuid in
+    let r = `Import from_uuid in
+    let* x = Api.(post (draft_trustees uuid) !user r) in
+    match x.code with
+    | 200 ->
+        let* () = send_draft_request @@ `SetTrusteesSetupStep 3 in
+        step := 3;
+        Cache.invalidate_all ();
+        !update_main_zone ()
+    | code ->
+        Printf.ksprintf alert "Failed with error code %d" code;
+        !update_main_zone ()
   in
   Lwt.return
     [
@@ -303,6 +304,8 @@ let recompute_main_zone_1 () =
       add_symbol;
       thresh;
       div ~a:[ a_id "trustee_proc_but" ] [ proc_but ];
+      h2 [ txt @@ s_ "Import trustees from another election" ];
+      div [ import_but ];
     ]
 
 (* FIXME: This step 3 is just a dumb, now, but in the future, it should be
@@ -363,23 +366,59 @@ let string_of_state st =
       | Some 7 -> "done"
       | _ -> assert false)
 
-let maillink_of_token prefix tk =
-  let uuid = get_current_uuid () in
-  let link =
-    match !mode with
-    | `Basic -> prefix ^ "/trustee/" ^ uuid ^ "/" ^ tk
-    | `Threshold _ -> prefix ^ "/threshold-trustee/" ^ uuid ^ "/" ^ tk
+module Mails = Belenios_ui.Mails_admin.Make (Belenios_js.I18n)
+
+let trustee_generate_link kind =
+  let generate_mail =
+    match kind with
+    | `Basic -> Mails.mail_trustee_generation_basic
+    | `Threshold -> Mails.mail_trustee_generation_threshold
   in
-  a ~href:link "Link"
+  fun ~token ~recipient ->
+    let open (val !Belenios_js.I18n.gettext) in
+    let uuid = get_current_uuid () in
+    let* prefix = Cache.get_prefix () in
+    let href =
+      Printf.sprintf "%strustee#generate/%s/%s" prefix (Uuid.unwrap uuid) token
+    in
+    let* subject, body = generate_mail [ lang ] href in
+    Lwt.return
+    @@ span
+         ~a:[ a_class [ "trustee-links" ] ]
+         [
+           a
+             ~a:[ a_class [ "trustee-link"; "trustee-generate-link" ] ]
+             ~href (s_ "Direct link");
+           br ();
+           a_mailto
+             ~a:[ a_class [ "trustee-link" ] ]
+             ~recipient ~subject ~body (s_ "Send an e-mail");
+         ]
 
 let all_ttee_done () =
   let dd = if !mode = `Basic then Some 1 else Some 7 in
   List.for_all (fun t -> t.trustee_state = dd) !all_trustee
 
-let maillink_of_token_direct prefix tk =
+let trustee_decrypt_link ~token ~recipient =
+  let open (val !Belenios_js.I18n.gettext) in
   let uuid = get_current_uuid () in
-  let link = prefix ^ "/trustees.html#" ^ uuid ^ "-" ^ tk in
-  a ~href:link "Link"
+  let* prefix = Cache.get_prefix () in
+  let href =
+    Printf.sprintf "%strustee#decrypt/%s/%s" prefix (Uuid.unwrap uuid) token
+  in
+  let* subject, body = Mails.mail_trustee_tally [ lang ] href in
+  Lwt.return
+  @@ span
+       ~a:[ a_class [ "trustee-links" ] ]
+       [
+         a
+           ~a:[ a_class [ "trustee-link"; "trustee-decrypt-link" ] ]
+           ~href (s_ "Direct link");
+         br ();
+         a_mailto
+           ~a:[ a_class [ "trustee-link" ] ]
+           ~recipient ~subject ~body (s_ "Send an e-mail");
+       ]
 
 let recompute_main_zone_2 () =
   let open (val !Belenios_js.I18n.gettext) in
@@ -400,8 +439,12 @@ let recompute_main_zone_2 () =
     step := 3;
     recompute_main_zone_3 ())
   else
-    let prefix = url_prefix () ^ "/draft" in
-    (* TODO: put also a mailto link *)
+    let trustee_generate_link =
+      let kind =
+        match !mode with `Basic -> `Basic | `Threshold _ -> `Threshold
+      in
+      trustee_generate_link kind
+    in
     let header_row =
       tr
         [
@@ -412,21 +455,23 @@ let recompute_main_zone_2 () =
           th [];
         ]
     in
-    let rows_of_ttees =
-      List.map
+    let* rows_of_ttees =
+      Lwt_list.map_s
         (fun t ->
+          let* link =
+            trustee_generate_link
+              ~token:(Option.value ~default:"" t.trustee_token)
+              ~recipient:(Option.value ~default:"" t.trustee_address)
+          in
           tr
             [
               td [ txt @@ Option.value ~default:"" t.trustee_address ];
               td [ txt t.trustee_name ];
-              td
-                [
-                  maillink_of_token prefix
-                    (match t.trustee_token with Some x -> x | None -> "");
-                ];
+              td [ link ];
               td [ txt @@ string_of_state t.trustee_state ];
               td [];
-            ])
+            ]
+          |> Lwt.return)
         !all_trustee
     in
     let reset_but =
@@ -445,6 +490,17 @@ let recompute_main_zone_2 () =
     Lwt.return
       [
         h2 [ txt @@ s_ "Trustee setup - Step 2: key generation" ];
+        div
+          [
+            em
+              [
+                txt
+                @@ s_
+                     "You must send each trustee their link (from the table \
+                      below) so that they initialize their key for this \
+                      election.";
+              ];
+          ];
         tablex [ tbody (header_row :: rows_of_ttees) ];
         div [ refresh_but ];
         div [ reset_but ];
@@ -456,9 +512,7 @@ let trustee_request req =
   Cache.invalidate Cache.e_status;
   let* status = Cache.get_until_success Cache.e_status in
   let ifmatch = Some (sha256_b64 @@ string_of_election_status status) in
-  let* x =
-    post_with_token ?ifmatch (string_of_admin_request req) "elections/%s" uuid
-  in
+  let* x = Api.(post ?ifmatch (election_status uuid) !user req) in
   match x.code with
   | 200 ->
       Cache.invalidate Cache.e_status;
@@ -494,9 +548,7 @@ let enough_pd () =
 
 let get_trustees_pd () =
   let uuid = get_current_uuid () in
-  let* x =
-    get partial_decryptions_of_string "elections/%s/partial-decryptions" uuid
-  in
+  let* x = Api.(get (election_partial_decryptions uuid) !user) in
   match x with
   | Error e ->
       alert (string_of_error e);
@@ -507,12 +559,11 @@ let get_trustees_pd () =
 
 let main_zone_tallying () =
   let open (val !Belenios_js.I18n.gettext) in
-  let content =
+  let* content =
     match !part_dec with
-    | None -> div [ txt @@ s_ "Failed to connect; please reload" ]
+    | None -> Lwt.return @@ div [ txt @@ s_ "Failed to connect; please reload" ]
     | Some x ->
         let tl = x.partial_decryptions_trustees in
-        let prefix = url_prefix () ^ "/election" in
         let header_row =
           tr
             [
@@ -522,18 +573,22 @@ let main_zone_tallying () =
               th [];
             ]
         in
-        let rows_of_ttees =
-          List.filter_map
+        let* rows_of_ttees =
+          Lwt_list.filter_map_s
             (fun t ->
-              if t.trustee_pd_address = "server" then None
+              if t.trustee_pd_address = "server" then Lwt.return_none
               else
-                Some
-                  (tr
+                let* link =
+                  trustee_decrypt_link ~token:t.trustee_pd_token
+                    ~recipient:t.trustee_pd_address
+                in
+                Lwt.return_some
+                @@ tr
                      [
                        td [ txt t.trustee_pd_address ];
-                       td [ maillink_of_token_direct prefix t.trustee_pd_token ];
+                       td [ link ];
                        td [ (txt @@ if t.trustee_pd_done then "yes" else "no") ];
-                     ]))
+                     ])
             tl
         in
         let refresh_but =
@@ -548,26 +603,44 @@ let main_zone_tallying () =
                 let* () = trustee_request `ReleaseTally in
                 !update_main_zone ())
         in
-        div
-          [
-            tablex [ tbody (header_row :: rows_of_ttees) ];
-            refresh_but;
-            release_but;
-          ]
+        Lwt.return
+        @@ div
+             [
+               div
+                 [
+                   em
+                     [
+                       txt
+                       @@ s_
+                            "You must send each trustee their link (from the \
+                             table below) so that they perform their share of \
+                             the decryption.";
+                     ];
+                 ];
+               tablex [ tbody (header_row :: rows_of_ttees) ];
+               refresh_but;
+               release_but;
+             ]
   in
   Lwt.return [ h2 [ txt @@ s_ "Tallying" ]; content ]
 
 let shuffles = ref None
 
-let get_shuffles () =
-  let uuid = get_current_uuid () in
-  let* x = get shuffles_of_string "elections/%s/shuffles" uuid in
+let get_shuffles uuid =
+  let* x = Api.(get (election_shuffles uuid) !user) in
   match x with
   | Error e ->
       alert (string_of_error e);
-      Lwt.return_unit
-  | Ok (tt, _) ->
-      shuffles := Some tt;
+      Lwt.return_none
+  | Ok (tt, _) -> Lwt.return_some tt
+
+let update_shuffles () =
+  let uuid = get_current_uuid () in
+  let* x = get_shuffles uuid in
+  match x with
+  | None -> Lwt.return_unit
+  | Some x ->
+      shuffles := Some x;
       Lwt.return_unit
 
 let ready_to_decrypt () =
@@ -579,20 +652,35 @@ let ready_to_decrypt () =
           t.shuffler_address = "server" || t.shuffler_fingerprint <> None)
         sh.shuffles_shufflers
 
-let shuffle_link prefix token =
+let trustee_shuffle_link ~token ~recipient =
+  let open (val !Belenios_js.I18n.gettext) in
   let uuid = get_current_uuid () in
-  let link = prefix ^ "/shuffle.html#" ^ uuid ^ "-" ^ token in
-  a ~href:link "Link"
+  let* prefix = Cache.get_prefix () in
+  let href =
+    Printf.sprintf "%strustee#shuffle/%s/%s" prefix (Uuid.unwrap uuid) token
+  in
+  let* subject, body = Mails.mail_shuffle [ lang ] href in
+  Lwt.return
+  @@ span
+       ~a:[ a_class [ "trustee-links" ] ]
+       [
+         a
+           ~a:[ a_class [ "trustee-link"; "trustee-shuffle-link" ] ]
+           ~href (s_ "Direct link");
+         br ();
+         a_mailto
+           ~a:[ a_class [ "trustee-link" ] ]
+           ~recipient ~subject ~body (s_ "Send an e-mail");
+       ]
 
 let main_zone_shuffling () =
   let open (val !Belenios_js.I18n.gettext) in
   let uuid = get_current_uuid () in
-  let content =
+  let* content =
     match !shuffles with
-    | None -> div [ txt @@ s_ "Failed to connect; please reload" ]
+    | None -> Lwt.return @@ div [ txt @@ s_ "Failed to connect; please reload" ]
     | Some x ->
         let sl = x.shuffles_shufflers in
-        let prefix = url_prefix () ^ "/election" in
         let header_row =
           tr
             [
@@ -608,11 +696,15 @@ let main_zone_shuffling () =
             (fun t -> match t.shuffler_token with Some _ -> true | _ -> false)
             sl
         in
-        let sel_but_list =
-          List.map
+        let* sel_but_list =
+          Lwt_list.map_s
             (fun t ->
               match t.shuffler_token with
-              | Some token -> [ shuffle_link prefix token ]
+              | Some token ->
+                  let* link =
+                    trustee_shuffle_link ~token ~recipient:t.shuffler_address
+                  in
+                  Lwt.return [ link ]
               | _ ->
                   let attr =
                     if sel_exists || t.shuffler_fingerprint <> None then
@@ -621,14 +713,11 @@ let main_zone_shuffling () =
                   in
                   let make_but lab req =
                     button ~a:attr lab (fun () ->
-                        let encoded =
-                          t.shuffler_address |> Js.string
-                          |> Js.encodeURIComponent |> Js.to_string
-                        in
                         let* x =
-                          post_with_token
-                            (string_of_shuffler_request req)
-                            "elections/%s/shuffles/%s" uuid encoded
+                          Api.(
+                            post
+                              (election_shuffle uuid t.shuffler_address)
+                              !user req)
                         in
                         match x.code with
                         | 200 -> !update_main_zone ()
@@ -638,7 +727,7 @@ let main_zone_shuffling () =
                   in
                   let but1 = make_but (s_ "Select this trustee") `Select in
                   let but2 = make_but (s_ "Skip") `Skip in
-                  [ but1; but2 ])
+                  Lwt.return [ but1; but2 ])
             sl
         in
         let rows_of_sh =
@@ -682,64 +771,100 @@ let main_zone_shuffling () =
               let* () = trustee_request `FinishShuffling in
               !update_main_zone ())
         in
-        div
-          [
-            tablex [ tbody (header_row :: rows_of_sh) ];
-            div [ refresh_but ];
-            div [ finish_but ];
-          ]
+        Lwt.return
+        @@ div
+             [
+               div
+                 [
+                   em
+                     [
+                       txt
+                       @@ s_
+                            "You must select each trustee in turn, and send \
+                             them a link so that they perform their shuffle. \
+                             You can also skip a trustee, but keep in mind \
+                             that this reduces security.";
+                     ];
+                 ];
+               tablex [ tbody (header_row :: rows_of_sh) ];
+               div [ refresh_but ];
+               div [ finish_but ];
+             ]
   in
   Lwt.return [ h2 [ txt @@ s_ "Tallying: shuffling step" ]; content ]
 
-let main_zone_tallied () =
-  let open (val !Belenios_js.I18n.gettext) in
-  let but =
-    button (s_ "Election main page") (fun () -> Preview.goto_mainpage ())
-  in
-  Lwt.return
-    [
-      h2 [ txt @@ s_ "This election has been tallied" ];
-      div [ txt @@ s_ "Go see the result on the election main page!" ];
-      div [ but ];
-    ]
-
 let recompute_main_zone () =
   let open (val !Belenios_js.I18n.gettext) in
-  if is_draft () then
-    let* () = get_trustees () in
-    let* content =
+  let checkpriv () =
+    let uuid = get_current_uuid () in
+    let href = "trustee#check/" ^ Uuid.unwrap uuid in
+    let label = s_ "Check private key ownership" in
+    [
+      h2 [ txt label ];
+      div
+        [
+          txt
+          @@ s_
+               "The following link can be used by trustees to check that they \
+                own the right decryption key.";
+        ];
+      ul [ li [ a ~href label ] ];
+    ]
+  in
+  let* content, show_checkpriv =
+    if is_draft () then (
+      let* () = get_trustees () in
+      let@ () =
+       fun cont ->
+        let* r = cont () in
+        Lwt.return (r, !step >= 3 && !all_trustee <> [])
+      in
       match !step with
       | 1 -> recompute_main_zone_1 ()
       | 2 -> recompute_main_zone_2 ()
       | 3 -> recompute_main_zone_3 ()
       | _ ->
           alert "Should not get there; aborting.";
-          assert false
-    in
-    Lwt.return content
-  else
-    let* status = Cache.get_until_success Cache.e_status in
-    let* content =
+          assert false)
+    else
+      let* status = Cache.get_until_success Cache.e_status in
       match status.status_state with
-      | `Tallied -> main_zone_tallied ()
       | `EncryptedTally ->
           let* () = get_trustees_pd () in
-          if not (all_pd ()) then main_zone_tallying ()
-          else
+          if all_pd () then
             let* () = trustee_request `ReleaseTally in
-            main_zone_tallied ()
-      | `Shuffling ->
-          let* () = get_shuffles () in
-          if not (ready_to_decrypt ()) then main_zone_shuffling ()
+            Lwt.return ([], false)
           else
+            let* r = main_zone_tallying () in
+            Lwt.return (r, false)
+      | `Shuffling ->
+          let* () = update_shuffles () in
+          if ready_to_decrypt () then
             let* () = trustee_request `FinishShuffling in
             let* () = get_trustees_pd () in
-            main_zone_tallying ()
-      | _ ->
-          Lwt.return
-            [ h2 [ txt @@ s_ "Tally" ]; div [ txt "Should not get there" ] ]
-    in
-    Lwt.return content
+            let* r = main_zone_tallying () in
+            Lwt.return (r, false)
+          else
+            let* r = main_zone_shuffling () in
+            Lwt.return (r, false)
+      | _ -> Lwt.return ([], false)
+  in
+  match content with
+  | [] ->
+      Lwt.return
+      @@ [
+           div
+             [
+               em
+                 [
+                   txt
+                   @@ s_ "Nothing to show here. Try going to the Status tab.";
+                 ];
+             ];
+         ]
+  | _ ->
+      let checkpriv = if show_checkpriv then checkpriv () else [] in
+      Lwt.return @@ List.flatten [ content; checkpriv ]
 
 let () =
   update_main_zone :=
