@@ -24,7 +24,7 @@ open Js_of_ocaml
 open Js_of_ocaml_tyxml
 open Belenios
 open Belenios_api.Serializable_j
-open Belenios_api.Common
+open Belenios_api
 open Tyxml_js.Html5
 open Belenios_js.Common
 open Belenios_js.Session
@@ -37,14 +37,16 @@ let show_draft_main show_all uuid draft container =
   let ifmatch = sha256_b64 draft_str in
   let button_save =
     let@ () = button "Save changes" in
-    let* x = put_with_token ~ifmatch (tget ()) "drafts/%s" uuid in
+    let* x =
+      Api.(put ~ifmatch (draft uuid) !user (draft_of_string (tget ())))
+    in
     let@ () = show_in container in
     generic_proceed x show_all
   in
   let button_delete =
     let@ () = button "Delete draft" in
     if confirm "Are you sure?" then (
-      let* x = delete_with_token ~ifmatch "drafts/%s" uuid in
+      let* x = Api.(delete (draft uuid) !user) in
       let@ () = show_in container in
       let@ () = generic_proceed x in
       Dom_html.window##.location##.hash := Js.string "";
@@ -55,13 +57,16 @@ let show_draft_main show_all uuid draft container =
 
 let rec show_draft_voters uuid draft container =
   let@ () = show_in container in
-  let* x = get voter_list_of_string "drafts/%s/voters" uuid in
+  let* x = Api.(get (draft_voters uuid) !user) in
   let@ voters, ifmatch = with_ok "voters" x in
   let voters_str = string_of_voter_list voters in
   let t, tget = textarea voters_str in
   let b =
     let@ () = button "Save changes" in
-    let* x = put_with_token ~ifmatch (tget ()) "drafts/%s/voters" uuid in
+    let* x =
+      Api.(
+        put ~ifmatch (draft_voters uuid) !user (voter_list_of_string (tget ())))
+    in
     let@ () = show_in container in
     generic_proceed x (fun () -> show_draft_voters uuid draft container)
   in
@@ -70,11 +75,7 @@ let rec show_draft_voters uuid draft container =
     let b =
       let@ () = button "Import voters" in
       let r = `Import (Uuid.wrap (iget ())) in
-      let* x =
-        post_with_token ~ifmatch
-          (string_of_voters_request r)
-          "drafts/%s/voters" uuid
-      in
+      let* x = Api.(post ~ifmatch (draft_voters uuid) !user r) in
       let@ () = show_in container in
       generic_proceed x (fun () -> show_draft_voters uuid draft container)
     in
@@ -84,9 +85,9 @@ let rec show_draft_voters uuid draft container =
 
 let rec show_draft_passwords uuid container =
   let@ () = show_in container in
-  let* x = get voter_list_of_string "drafts/%s/voters" uuid in
+  let* x = Api.(get (draft_voters uuid) !user) in
   let@ voters, _ = with_ok "voters" x in
-  let* x = get string_list_of_string "drafts/%s/passwords" uuid in
+  let* x = Api.(get (draft_passwords uuid) !user) in
   let@ x, ifmatch = with_ok "passwords" x in
   let missing =
     let x =
@@ -104,7 +105,11 @@ let rec show_draft_passwords uuid container =
   let t2, t2get = textarea (string_of_string_list missing) in
   let b =
     let@ () = button "Generate and send passwords" in
-    let* x = post_with_token ~ifmatch (t2get ()) "drafts/%s/passwords" uuid in
+    let* x =
+      Api.(
+        post ~ifmatch (draft_passwords uuid) !user
+          (voter_list_of_string (t2get ())))
+    in
     let@ () = show_in container in
     generic_proceed x (fun () -> show_draft_passwords uuid container)
   in
@@ -112,20 +117,17 @@ let rec show_draft_passwords uuid container =
 
 let rec show_draft_credentials uuid container =
   let@ () = show_in container in
-  let* x =
-    get public_credentials_of_string "drafts/%s/credentials/public" uuid
-  in
+  let* x = Api.(get (draft_public_credentials uuid) !user) in
   let@ x = with_ok_not_found "credentials" x in
   match x with
   | None -> (
-      let* x = get (fun x -> x) "drafts/%s/credentials/token" uuid in
+      let* x = Api.(get (draft_credentials_token uuid) !user) in
       let@ x = with_ok_not_found "token" x in
       match x with
       | None ->
           let b =
             let@ () = button "Generate on server" in
-            let op = string_of_public_credentials [] in
-            let* x = post_with_token op "drafts/%s/credentials" uuid in
+            let* x = Api.(post (draft_public_credentials uuid) !user []) in
             let@ () = show_in container in
             generic_proceed x (fun () -> show_draft_credentials uuid container)
           in
@@ -156,12 +158,12 @@ type trustee_with_writer =
 
 let rec show_draft_trustees uuid container =
   let@ () = show_in container in
-  let* x =
-    get
-      (draft_trustees_of_string Yojson.Safe.read_json Yojson.Safe.read_json)
-      "drafts/%s/trustees" uuid
-  in
+  let* x = Api.(get (draft_trustees uuid) !user) in
   let@ trustees, ifmatch = with_ok "trustees" x in
+  let trustees =
+    draft_trustees_of_string Yojson.Safe.read_json Yojson.Safe.read_json
+      trustees
+  in
   let mode =
     match trustees with
     | `Basic _ -> "basic"
@@ -187,11 +189,7 @@ let rec show_draft_trustees uuid container =
             alert "Unrecognized mode";
             Lwt.return_unit
       in
-      let* x =
-        post_with_token ~ifmatch
-          (string_of_trustees_request request)
-          "drafts/%s/trustees" uuid
-      in
+      let* x = Api.(post ~ifmatch (draft_trustees uuid) !user request) in
       let@ () = show_in container in
       generic_proceed x (fun () -> show_draft_trustees uuid container)
     in
@@ -209,16 +207,11 @@ let rec show_draft_trustees uuid container =
   let all_trustees =
     List.map
       (fun t ->
-        let encoded_trustee =
-          Option.value ~default:"@" t.trustee_address
-          |> Js.string |> Js.encodeURIComponent |> Js.to_string
-        in
+        let trustee_address = Option.value ~default:"@" t.trustee_address in
         let content =
           let b =
             let@ () = button "Delete" in
-            let* x =
-              delete_with_token "drafts/%s/trustees/%s" uuid encoded_trustee
-            in
+            let* x = Api.(delete (draft_trustee uuid trustee_address) !user) in
             let@ () = show_in container in
             generic_proceed x (fun () -> show_draft_trustees uuid container)
           in
@@ -232,11 +225,7 @@ let rec show_draft_trustees uuid container =
   let b =
     let@ () = button "Add trustee" in
     let r = `Add (trustee_of_string Yojson.Safe.read_json (t2get ())) in
-    let* x =
-      post_with_token ~ifmatch
-        (string_of_trustees_request r)
-        "drafts/%s/trustees" uuid
-    in
+    let* x = Api.(post ~ifmatch (draft_trustees uuid) !user r) in
     let@ () = show_in container in
     generic_proceed x (fun () -> show_draft_trustees uuid container)
   in
@@ -245,11 +234,7 @@ let rec show_draft_trustees uuid container =
     let b =
       let@ () = button "Import trustees" in
       let r = `Import (Uuid.wrap (iget ())) in
-      let* x =
-        post_with_token ~ifmatch
-          (string_of_trustees_request r)
-          "drafts/%s/trustees" uuid
-      in
+      let* x = Api.(post ~ifmatch (draft_trustees uuid) !user r) in
       let@ () = show_in container in
       generic_proceed x (fun () -> show_draft_trustees uuid container)
     in
@@ -260,17 +245,17 @@ let rec show_draft_trustees uuid container =
 
 let rec show_draft_status uuid container =
   let@ () = show_in container in
-  let* x = get draft_status_of_string "drafts/%s/status" uuid in
+  let* x = Api.(get (draft_status uuid) !user) in
   let@ status, _ = with_ok "status" x in
   let t, _ = textarea (string_of_draft_status status) in
   let b label r =
     let@ () = button label in
-    let* x = post_with_token (string_of_draft_request r) "drafts/%s" uuid in
+    let* x = Api.(post (draft uuid) !user r) in
     let@ () = show_in container in
     let@ () = generic_proceed x in
     match (r, x.code) with
     | `ValidateElection, 200 ->
-        let new_hash = Printf.sprintf "#elections/%s" uuid in
+        let new_hash = Printf.sprintf "#elections/%s" (Uuid.unwrap uuid) in
         Dom_html.window##.location##.hash := Js.string new_hash;
         Lwt.return_unit
     | _ -> show_draft_status uuid container
@@ -309,18 +294,18 @@ let show_draft show_all uuid draft title container tab =
 
 let a_draft_tab uuid tab =
   let suffix, label = suffix_and_label_of_draft_tab tab in
-  let href = Printf.sprintf "#drafts/%s%s" uuid suffix in
+  let href = Printf.sprintf "#drafts/%s%s" (Uuid.unwrap uuid) suffix in
   a ~href label
 
 let show main uuid tab context =
   let rec show_all () =
-    let* x = get draft_of_string "drafts/%s" uuid in
+    let* x = Api.(get (draft uuid) `Nobody) in
     match x with
     | Error e ->
         let@ () = show_in main in
         let msg =
-          Printf.sprintf "An error occurred while retrieving draft %s: %s" uuid
-            (string_of_error e)
+          Printf.sprintf "An error occurred while retrieving draft %s: %s"
+            (Uuid.unwrap uuid) (string_of_error e)
         in
         Lwt.return [ h1 [ txt "Error" ]; div [ txt msg ] ]
     | Ok ((Draft (_, d) as draft), _) ->
