@@ -20,65 +20,51 @@
 (**************************************************************************)
 
 open Lwt.Syntax
-open Lwt.Infix
 open Belenios
-open Web_common
-open Web_serializable_j
+open Belenios_server_core
 
-module UMap = Map.Make (struct
-  type t = user
-
-  let compare = compare
-end)
-
-let cache = ref None
-let counter_mutex = Lwt_mutex.create ()
-let account_mutex = Lwt_mutex.create ()
-let cache_mutex = Lwt_mutex.create ()
-
-let clear_account_cache () =
-  let@ () = Lwt_mutex.with_lock cache_mutex in
-  cache := None;
-  Lwt.return_unit
-
-let account_of_filename filename =
-  let&* id = Filename.chop_suffix_opt ~suffix:".json" filename in
-  let&* _ = int_of_string_opt id in
-  let* contents = Filesystem.read_file (!Web_config.accounts_dir // filename) in
-  match contents with
-  | Some [ x ] -> Lwt.return (try Some (account_of_string x) with _ -> None)
-  | _ -> Lwt.return_none
-
-let get_account_by_id id = account_of_filename (Printf.sprintf "%d.json" id)
 let update_hooks = ref []
 let add_update_hook f = update_hooks := f :: !update_hooks
 
-let update_account account =
-  let* () =
-    let@ () = Lwt_mutex.with_lock account_mutex in
-    Filesystem.write_file
-      (!Web_config.accounts_dir // Printf.sprintf "%d.json" account.id)
-      [ string_of_account account ]
-  in
+let run_update_hooks account =
   Lwt_list.iter_s (fun f -> f account) !update_hooks
 
+let get_account_by_id s id =
+  let module S = (val s : Storage.BACKEND) in
+  let* x = S.get (Account id) in
+  match x with
+  | None -> Lwt.return_none
+  | Some x -> (
+      match account_of_string x with
+      | exception _ -> Lwt.return_none
+      | x -> Lwt.return_some x)
+
+let update_account_by_id s id =
+  let module S = (val s : Storage.BACKEND) in
+  let* x = S.update (Account id) in
+  match x with
+  | None -> Lwt.return_none
+  | Some (x, set) -> (
+      let set x =
+        let* () = set (string_of_account x) in
+        run_update_hooks x
+      in
+      match account_of_string x with
+      | exception _ -> Lwt.return_none
+      | x -> Lwt.return (Some (x, set)))
+
 let drop_after_at x =
   match String.index_opt x '@' with None -> x | Some i -> String.sub x 0 i
 
-let create_account ~email user =
-  let@ () = Lwt_mutex.with_lock counter_mutex in
-  let* counter =
-    let* x = Filesystem.read_file (!Web_config.accounts_dir // "counter") in
+let create_account s ~email user =
+  let module S = (val s : Storage.BACKEND) in
+  let@ id, u =
+   fun cont ->
+    let* x = S.new_account_id () in
     match x with
-    | Some [ x ] ->
-        Lwt.return (match int_of_string_opt x with None -> 1 | Some x -> x)
-    | _ -> Lwt.return 1
+    | None -> Lwt.fail (Failure "impossible to create a new account")
+    | Some x -> cont x
   in
-  let rec find_free_id n =
-    let* x = get_account_by_id n in
-    match x with None -> Lwt.return n | Some _ -> find_free_id (n + 1)
-  in
-  let* id = find_free_id counter in
   let last_connected = Datetime.now () in
   let name =
     let x = drop_after_at user.user_name in
@@ -99,43 +85,17 @@ let create_account ~email user =
       voters_limit = None;
     }
   in
-  let* () = update_account account in
   let* () =
-    Filesystem.write_file
-      (!Web_config.accounts_dir // "counter")
-      [ string_of_int (id + 1) ]
+    Lwt.finalize
+      (fun () ->
+        let* () = S.create (Account id) (string_of_account account) in
+        run_update_hooks account)
+      (fun () ->
+        Lwt.wakeup_later u ();
+        Lwt.return_unit)
   in
-  let* () = clear_account_cache () in
   Lwt.return account
 
-let build_account_cache () =
-  Lwt_unix.files_of_directory !Web_config.accounts_dir
-  |> Lwt_stream.to_list
-  >>= Lwt_list.fold_left_s
-        (fun accu f ->
-          let* account = account_of_filename f in
-          match account with
-          | None -> Lwt.return accu
-          | Some account ->
-              List.fold_left
-                (fun accu u -> UMap.add u account.id accu)
-                accu account.authentications
-              |> Lwt.return)
-        UMap.empty
-
-let get_account user =
-  let* cache =
-    match !cache with
-    | Some x -> Lwt.return x
-    | None ->
-        let@ () = Lwt_mutex.with_lock cache_mutex in
-        let* x = build_account_cache () in
-        cache := Some x;
-        Lwt.return x
-  in
-  let&* id = UMap.find_opt user cache in
-  get_account_by_id id
-
 type capability = Sudo
 
 let mask_of_capability = function Sudo -> 1
