@@ -22,8 +22,8 @@
 open Lwt.Syntax
 open Belenios
 open Belenios_api.Serializable_j
-open Belenios_api.Common
-open Web_serializable_j
+open Belenios_api
+open Belenios_server_core
 open Web_common
 open Api_generic
 
@@ -49,27 +49,34 @@ let with_administrator_or_nobody token (Draft (_, se)) f =
       | Some a when Accounts.check a se.se_owners -> f (`Administrator a)
       | _ -> not_found)
 
-let with_threshold_trustee token (Draft (_, se)) f =
+let with_trustee token (Draft (_, se)) f =
   let@ token = Option.unwrap unauthorized token in
   match se.se_trustees with
-  | `Basic _ -> not_found
+  | `Basic b -> (
+      match List.find_opt (fun x -> x.st_token = token) b.dbp_trustees with
+      | Some x -> f (`Basic x)
+      | None -> unauthorized)
   | `Threshold t -> (
       match
         List.findi
           (fun i x -> if x.stt_token = token then Some (i, x) else None)
           t.dtp_trustees
       with
-      | Some (i, x) -> f (i + 1, x, t)
-      | None -> not_found)
+      | Some (i, x) -> f (`Threshold (i + 1, x, t))
+      | None -> unauthorized)
+
+let authentication_of_auth_config = function
+  | Some [ { auth_system = "password"; _ } ] -> Some `Password
+  | Some [ { auth_system = "cas"; auth_config; _ } ] ->
+      Some (`CAS (List.assoc "server" auth_config))
+  | Some [ { auth_system = "import"; auth_instance; _ } ] ->
+      Some (`Configured auth_instance)
+  | _ -> None
 
 let get_authentication se =
-  match se.se_metadata.e_auth_config with
-  | Some [ { auth_system = "password"; _ } ] -> `Password
-  | Some [ { auth_system = "cas"; auth_config; _ } ] ->
-      `CAS (List.assoc "server" auth_config)
-  | Some [ { auth_system = "import"; auth_instance; _ } ] ->
-      `Configured auth_instance
-  | _ -> raise (Error (`Invalid "authentication"))
+  match authentication_of_auth_config se.se_metadata.e_auth_config with
+  | Some x -> x
+  | None -> raise (Error (`Invalid "authentication"))
 
 let auth_config_of_authentication = function
   | `Password ->
@@ -93,7 +100,7 @@ let api_of_draft (Draft (v, se)) =
     }
   in
   Lwt.return
-    (Belenios_api.Common.Draft
+    (Belenios_api.Draft
        ( v,
          {
            draft_version = se.se_version;
@@ -108,8 +115,7 @@ let api_of_draft (Draft (v, se)) =
 
 let assert_ msg b f = if b then f () else raise (Error msg)
 
-let draft_of_api a uuid (Draft (v, se) as fse)
-    (Belenios_api.Common.Draft (v', d)) =
+let draft_of_api a uuid (Draft (v, se) as fse) (Belenios_api.Draft (v', d)) =
   let version = se.se_version in
   let@ Refl =
    fun cont ->
@@ -119,7 +125,7 @@ let draft_of_api a uuid (Draft (v, se) as fse)
   in
   let@ () =
     assert_ (`Invalid "booth version")
-      (List.mem d.draft_booth Web_defaults.supported_booth_versions)
+      (List.mem d.draft_booth Defaults.supported_booth_versions)
   in
   let@ () =
     assert_ (`Invalid "languages") (List.length d.draft_languages >= 1)
@@ -176,19 +182,11 @@ let draft_of_api a uuid (Draft (v, se) as fse)
   }
   |> fun x -> Draft (v, x)
 
-let generate_uuid () =
-  let length = !Web_config.uuid_length in
-  let token = generate_token ?length () in
-  Lwt.return (Uuid.wrap token)
-
-let post_drafts account draft =
+let post_drafts account s draft =
   let@ () =
-   fun cont ->
-    if !Web_config.deny_newelection then Lwt.return_none
-    else Lwt.(cont () >>= return_some)
+   fun cont -> if !Web_config.deny_newelection then Lwt.return_none else cont ()
   in
   let owners = [ account.id ] in
-  let* uuid = generate_uuid () in
   let token = generate_token () in
   let se_metadata =
     {
@@ -211,7 +209,7 @@ let post_drafts account draft =
       t_credential_authority = None;
     }
   in
-  let se_version = Web_defaults.version in
+  let se_version = Defaults.version in
   let (Version v) = Belenios.Election.version_of_int se_version in
   let se =
     {
@@ -232,14 +230,17 @@ let post_drafts account draft =
       se_pending_credentials = false;
     }
   in
+  let module S = (val s : Storage.BACKEND) in
+  let* uuid = S.new_election () in
+  let&* uuid = uuid in
   let se = draft_of_api account uuid (Draft (v, se)) draft in
-  let* () = Web_persist.create_draft uuid se in
-  Lwt.return uuid
+  let* () = Web_persist.create_draft s uuid se in
+  Lwt.return_some uuid
 
 let get_draft_voters (Draft (_, se)) =
   se.se_voters |> List.map (fun x -> x.sv_id)
 
-let put_draft_voters uuid (Draft (v, se)) voters =
+let put_draft_voters (Draft (v, se), set) voters =
   let existing_voters =
     List.fold_left
       (fun accu v ->
@@ -296,7 +297,7 @@ let put_draft_voters uuid (Draft (v, se)) voters =
     else Lwt.return_unit
   in
   let se = { se with se_voters } in
-  Web_persist.set_draft_election uuid (Draft (v, se))
+  set (Draft (v, se))
 
 let get_draft_passwords (Draft (_, se)) =
   se.se_voters
@@ -307,7 +308,7 @@ let get_draft_passwords (Draft (_, se)) =
              login)
            x.sv_password)
 
-let post_draft_passwords account generate uuid (Draft (v, se)) voters =
+let post_draft_passwords account generate (Draft (v, se), set) voters =
   let se_voters =
     List.fold_left
       (fun accu v ->
@@ -335,7 +336,7 @@ let post_draft_passwords account generate uuid (Draft (v, se)) voters =
         Lwt.return (job :: jobs))
       [] voters
   in
-  let* () = Web_persist.set_draft_election uuid (Draft (v, se)) in
+  let* () = set (Draft (v, se)) in
   Lwt.return jobs
 
 let get_credentials_token (Draft (_, se)) =
@@ -364,7 +365,7 @@ let exn_of_generate_credentials_on_server_error = function
   | `Already -> Error (`GenericError "already done")
   | `NoServer -> Error (`GenericError "credential authority is not the server")
 
-let submit_public_credentials uuid (Draft (v, se)) credentials =
+let submit_public_credentials s uuid (Draft (v, se), set) credentials =
   let () =
     if se.se_voters = [] then raise (Error (`ValidationError `NoVoters))
   in
@@ -421,12 +422,13 @@ let submit_public_credentials uuid (Draft (v, se)) credentials =
       credentials
   in
   let salts = List.rev salts in
-  let* () = Web_persist.set_draft_public_credentials uuid credentials in
+  let* () = Spool.create s uuid Spool.draft_public_credentials credentials in
   let* () =
-    if salts <> [] then Web_persist.set_salts uuid salts else Lwt.return_unit
+    if salts <> [] then Spool.create s uuid Spool.salts salts
+    else Lwt.return_unit
   in
   se.se_public_creds_received <- true;
-  Web_persist.set_draft_election uuid (Draft (v, se))
+  set (Draft (v, se))
 
 let get_draft_trustees ~is_admin (Draft (_, se)) =
   match se.se_trustees with
@@ -504,7 +506,7 @@ let generate_server_trustee (Draft (_, se)) =
   let st_name = Some "server" in
   Lwt.return { st_id; st_token; st_public_key; st_private_key; st_name }
 
-let post_draft_trustees uuid (Draft (v, se)) t =
+let post_draft_trustees (Draft (v, se), set) t =
   let address =
     match t.trustee_address with
     | Some x ->
@@ -539,7 +541,7 @@ let post_draft_trustees uuid (Draft (v, se)) t =
         }
       in
       x.dbp_trustees <- ts @ [ t ];
-      Web_persist.set_draft_election uuid (Draft (v, se))
+      set (Draft (v, se))
   | `Threshold x ->
       let ts = x.dtp_trustees in
       let () =
@@ -560,7 +562,7 @@ let post_draft_trustees uuid (Draft (v, se)) t =
         }
       in
       x.dtp_trustees <- ts @ [ t ];
-      Web_persist.set_draft_election uuid (Draft (v, se))
+      set (Draft (v, se))
 
 let rec filter_out_first f = function
   | [] -> (false, [])
@@ -570,14 +572,14 @@ let rec filter_out_first f = function
         let touched, xs = filter_out_first f xs in
         (touched, x :: xs)
 
-let delete_draft_trustee uuid (Draft (v, se)) trustee =
+let delete_draft_trustee (Draft (v, se), set) trustee =
   match se.se_trustees with
   | `Basic x ->
       let ts = x.dbp_trustees in
       let touched, ts = filter_out_first (fun x -> x.st_id = trustee) ts in
       if touched then (
         x.dbp_trustees <- ts;
-        let* () = Web_persist.set_draft_election uuid (Draft (v, se)) in
+        let* () = set (Draft (v, se)) in
         Lwt.return_true)
       else Lwt.return_false
   | `Threshold x ->
@@ -585,11 +587,11 @@ let delete_draft_trustee uuid (Draft (v, se)) trustee =
       let touched, ts = filter_out_first (fun x -> x.stt_id = trustee) ts in
       if touched then (
         x.dtp_trustees <- ts;
-        let* () = Web_persist.set_draft_election uuid (Draft (v, se)) in
+        let* () = set (Draft (v, se)) in
         Lwt.return_true)
       else Lwt.return_false
 
-let set_threshold uuid (Draft (v, se)) threshold =
+let set_threshold (Draft (v, se), set) threshold =
   match se.se_trustees with
   | `Basic _ -> Lwt.return @@ Stdlib.Error `NoTrustees
   | `Threshold x when x.dtp_trustees = [] ->
@@ -602,7 +604,7 @@ let set_threshold uuid (Draft (v, se)) threshold =
       if 0 <= threshold && threshold < List.length ts then (
         List.iter (fun t -> t.stt_step <- step) ts;
         x.dtp_threshold <- maybe_threshold;
-        let* () = Web_persist.set_draft_election uuid (Draft (v, se)) in
+        let* () = set (Draft (v, se)) in
         Lwt.return @@ Ok ())
       else Lwt.return @@ Stdlib.Error `OutOfBounds
 
@@ -611,12 +613,12 @@ let get_draft_trustees_mode (Draft (_, se)) =
   | `Basic _ -> `Basic
   | `Threshold x -> `Threshold (Option.value x.dtp_threshold ~default:0)
 
-let put_draft_trustees_mode uuid (Draft (v, se)) mode =
+let put_draft_trustees_mode (Draft (v, se), set) mode =
   match (get_draft_trustees_mode (Draft (v, se)), mode) with
   | a, b when a = b -> Lwt.return_unit
   | _, `Basic ->
       se.se_trustees <- `Basic { dbp_trustees = [] };
-      Web_persist.set_draft_election uuid (Draft (v, se))
+      set (Draft (v, se))
   | `Basic, `Threshold 0 ->
       let dtp =
         {
@@ -627,9 +629,9 @@ let put_draft_trustees_mode uuid (Draft (v, se)) mode =
         }
       in
       se.se_trustees <- `Threshold dtp;
-      Web_persist.set_draft_election uuid (Draft (v, se))
+      set (Draft (v, se))
   | `Threshold _, `Threshold threshold -> (
-      let* x = set_threshold uuid (Draft (v, se)) threshold in
+      let* x = set_threshold (Draft (v, se), set) threshold in
       match x with
       | Ok () -> Lwt.return_unit
       | Error `NoTrustees -> Lwt.fail (Error (`GenericError "no trustees"))
@@ -637,10 +639,10 @@ let put_draft_trustees_mode uuid (Draft (v, se)) mode =
           Lwt.fail (Error (`GenericError "threshold out of bounds")))
   | _, _ -> Lwt.fail (Error (`GenericError "change not allowed"))
 
-let get_draft_status uuid (Draft (v, se)) =
+let get_draft_status s uuid (Draft (v, se)) =
   let* private_credentials_downloaded =
     if se.se_metadata.e_cred_authority = Some "server" then
-      let* b = Web_persist.get_private_creds_downloaded uuid in
+      let* b = Web_persist.get_private_creds_downloaded s uuid in
       Lwt.return_some b
     else Lwt.return_none
   in
@@ -723,9 +725,36 @@ let merge_voters a b f =
   in
   loop weights (List.rev a) b
 
-let import_voters uuid (Draft (v, se)) from =
-  let* voters = Web_persist.get_all_voters from in
-  let* passwords = Web_persist.get_passwords from in
+let get_passwords s uuid =
+  let module S = (val s : Storage.BACKEND) in
+  let* csv = S.get (Election (uuid, Passwords)) in
+  let&* csv = csv in
+  let* csv =
+    Lwt_preemptive.detach (fun db -> Csv.(input_all (of_string db))) csv
+  in
+  let res =
+    List.fold_left
+      (fun accu line ->
+        match line with
+        | [ login; salt; hash ] ->
+            SMap.add (String.lowercase_ascii login) (salt, hash) accu
+        | _ -> accu)
+      SMap.empty csv
+  in
+  Lwt.return_some res
+
+let import_voters s uuid (Draft (v, se), set) from =
+  let@ voters cont =
+    let* x = Web_persist.get_all_voters s from in
+    match x with
+    | [] -> (
+        let* se = Spool.get s from Spool.draft in
+        match se with
+        | None -> Lwt.return @@ Stdlib.Error `NotFound
+        | Some se -> cont @@ get_draft_voters se)
+    | _ -> cont x
+  in
+  let* passwords = get_passwords s from in
   let get_password =
     match passwords with
     | None -> fun _ -> None
@@ -742,18 +771,18 @@ let import_voters uuid (Draft (v, se)) from =
         let expanded = Weight.expand ~total:total_weight total_weight in
         if Z.compare expanded Weight.max_expanded_weight <= 0 then (
           se.se_voters <- voters;
-          let* () = Web_persist.set_draft_election uuid (Draft (v, se)) in
+          let* () = set (Draft (v, se)) in
           Lwt.return @@ Ok ())
         else Lwt.return @@ Stdlib.Error (`TotalWeightTooBig total_weight)
     | Error x ->
         let _, login, _ = Voter.get x in
         Lwt.return @@ Stdlib.Error (`Duplicate login)
 
-let import_trustees uuid (Draft (v, se)) from metadata =
+let import_trustees (Draft (v, se), set) s from metadata =
   match metadata.e_trustees with
   | None -> Lwt.return @@ Stdlib.Error `None
   | Some names -> (
-      let* trustees = Web_persist.get_trustees from in
+      let* trustees = Public_archive.get_trustees s from in
       let version = se.se_version in
       let module G = (val Group.of_string ~version se.se_group : GROUP) in
       let module Trustees = (val Trustees.get_by_version version) in
@@ -764,13 +793,14 @@ let import_trustees uuid (Draft (v, se)) from metadata =
       if not (K.check trustees) then Lwt.return @@ Stdlib.Error `Invalid
       else
         let import_pedersen t names =
-          let* privs = Web_persist.get_private_keys from in
+          let* privs = Spool.get s from Spool.private_keys in
           let* x =
             match privs with
             | Some privs ->
-                let rec loop ts pubs privs accu =
-                  match (ts, pubs, privs) with
+                let rec loop ts certs pubs privs accu =
+                  match (ts, certs, pubs, privs) with
                   | ( stt_id :: ts,
+                      cert :: certs,
                       (vo_public_key : _ trustee_public_key) :: pubs,
                       vo_private_key :: privs ) ->
                       let stt_name = vo_public_key.trustee_name in
@@ -787,21 +817,30 @@ let import_trustees uuid (Draft (v, se)) from metadata =
                           stt_token;
                           stt_voutput;
                           stt_step = Some 7;
-                          stt_cert = None;
+                          stt_cert = Some cert;
                           stt_polynomial = None;
                           stt_vinput = None;
                           stt_name;
                         }
                       in
-                      loop ts pubs privs (stt :: accu)
-                  | [], [], [] -> Lwt.return @@ Ok (List.rev accu)
-                  | _, _, _ -> Lwt.return @@ Stdlib.Error `Inconsistent
+                      loop ts certs pubs privs (stt :: accu)
+                  | [], [], [], [] -> Lwt.return @@ Ok (List.rev accu)
+                  | _ -> Lwt.return @@ Stdlib.Error `Inconsistent
                 in
-                loop names (Array.to_list t.t_verification_keys) privs []
+                loop names (Array.to_list t.t_certs)
+                  (Array.to_list t.t_verification_keys)
+                  privs []
             | None -> Lwt.return @@ Stdlib.Error `MissingPrivateKeys
           in
           match x with
           | Ok se_threshold_trustees ->
+              let se_threshold_trustees =
+                se_threshold_trustees
+                |> List.map
+                     (string_of_draft_threshold_trustee (swrite G.Zq.to_string)
+                     >> draft_threshold_trustee_of_string Yojson.Safe.read_json
+                     )
+              in
               let dtp =
                 {
                   dtp_threshold = Some t.t_threshold;
@@ -814,7 +853,7 @@ let import_trustees uuid (Draft (v, se)) from metadata =
                 }
               in
               se.se_trustees <- `Threshold dtp;
-              let* () = Web_persist.set_draft_election uuid (Draft (v, se)) in
+              let* () = set (Draft (v, se)) in
               Lwt.return @@ Ok `Threshold
           | Stdlib.Error _ as x -> Lwt.return x
         in
@@ -867,29 +906,31 @@ let import_trustees uuid (Draft (v, se)) from metadata =
                        })
             in
             se.se_trustees <- `Basic { dbp_trustees = ts };
-            let* () = Web_persist.set_draft_election uuid (Draft (v, se)) in
+            let* () = set (Draft (v, se)) in
             Lwt.return @@ Ok `Basic)
 
-let check_owner account uuid cont =
-  let* metadata = Web_persist.get_election_metadata uuid in
+let check_owner account s uuid cont =
+  let* metadata = Web_persist.get_election_metadata s uuid in
   if Accounts.check account metadata.e_owners then cont metadata
   else unauthorized
 
-let post_draft_status ~admin_id uuid (Draft (v, se)) = function
+let post_draft_status ~admin_id s uuid (Draft (v, se), set) = function
   | `SetDownloaded ->
-      let* () = Web_persist.set_private_creds_downloaded uuid in
+      let* () = Web_persist.set_private_creds_downloaded s uuid in
       ok
   | `ValidateElection ->
-      let* s = get_draft_status uuid (Draft (v, se)) in
+      let* status = get_draft_status s uuid (Draft (v, se)) in
       let* () =
-        Web_persist.validate_election ~admin_id uuid (Draft (v, se)) s
+        Web_persist.validate_election ~admin_id s uuid
+          (Draft (v, se), set)
+          status
       in
       ok
   | `SetCredentialAuthorityVisited ->
       let* () =
         if se.se_credential_authority_visited <> true then (
           se.se_credential_authority_visited <- true;
-          let* () = Web_persist.set_draft_election uuid (Draft (v, se)) in
+          let* () = set (Draft (v, se)) in
           Lwt.return_unit)
         else Lwt.return_unit
       in
@@ -898,7 +939,7 @@ let post_draft_status ~admin_id uuid (Draft (v, se)) = function
       let* () =
         if se.se_voter_authentication_visited <> true then (
           se.se_voter_authentication_visited <- true;
-          let* () = Web_persist.set_draft_election uuid (Draft (v, se)) in
+          let* () = set (Draft (v, se)) in
           Lwt.return_unit)
         else Lwt.return_unit
       in
@@ -907,13 +948,172 @@ let post_draft_status ~admin_id uuid (Draft (v, se)) = function
       let* () =
         if se.se_trustees_setup_step <> i then (
           se.se_trustees_setup_step <- i;
-          let* () = Web_persist.set_draft_election uuid (Draft (v, se)) in
+          let* () = set (Draft (v, se)) in
           Lwt.return_unit)
         else Lwt.return_unit
       in
       ok
 
-let dispatch_credentials ~token endpoint method_ body uuid se =
+let post_trustee_basic ((Draft (_, se) as fse), set) ~token data =
+  let ts =
+    match se.se_trustees with
+    | `Basic x -> x.dbp_trustees
+    | `Threshold _ -> failwith "Wrong trustee mode"
+  in
+  let t =
+    match List.find_opt (fun x -> token = x.st_token) ts with
+    | Some t -> t
+    | None -> failwith "Invalid token"
+  in
+  match t.st_public_key with
+  | "" ->
+      let version = se.se_version in
+      let module G = (val Group.of_string ~version se.se_group : GROUP) in
+      let module Trustees = (val Trustees.get_by_version version) in
+      let pk =
+        trustee_public_key_of_string (sread G.of_string) (sread G.Zq.of_string)
+          data
+      in
+      let module K = Trustees.MakeCombinator (G) in
+      if K.check [ `Single pk ] then (
+        t.st_public_key <-
+          string_of_trustee_public_key (swrite G.to_string)
+            (swrite G.Zq.to_string) pk;
+        set fse)
+      else raise @@ Error `InvalidPublicKey
+  | _ -> raise @@ Error `PublicKeyExists
+
+let post_trustee_threshold ((Draft (_, se) as fse), set) ~token data =
+  let version = se.se_version in
+  let module G = (val Group.of_string ~version se.se_group : GROUP) in
+  let se_trustees =
+    se.se_trustees
+    |> string_of_draft_trustees Yojson.Safe.write_json
+    |> draft_trustees_of_string (sread G.Zq.of_string)
+  in
+  let dtp =
+    match se_trustees with
+    | `Basic _ -> failwith "Wrong trustee mode"
+    | `Threshold x -> x
+  in
+  let ts = Array.of_list dtp.dtp_trustees in
+  let threshold =
+    match dtp.dtp_threshold with
+    | Some t -> t
+    | None -> failwith "No threshold set"
+  in
+  let i, t =
+    match
+      Array.findi
+        (fun i x -> if token = x.stt_token then Some (i, x) else None)
+        ts
+    with
+    | Some (i, t) -> (i, t)
+    | None -> failwith "Trustee not found"
+  in
+  let context =
+    { group = se.se_group; size = Array.length ts; threshold; index = i + 1 }
+  in
+  let get_certs () =
+    Array.map
+      (fun x ->
+        match x.stt_cert with
+        | None -> failwith "Missing certificate"
+        | Some y -> y)
+      ts
+  in
+  let get_polynomials () =
+    Array.map
+      (fun x ->
+        match x.stt_polynomial with
+        | None -> failwith "Missing polynomial"
+        | Some y -> y)
+      ts
+  in
+  let module Trustees = (val Trustees.get_by_version version) in
+  let module P = Trustees.MakePKI (G) (Random) in
+  let module C = Trustees.MakeChannels (G) (Random) (P) in
+  let module K = Trustees.MakePedersen (G) (Random) (P) (C) in
+  let () =
+    match t.stt_step with
+    | Some 1 ->
+        let cert = cert_of_string (sread G.Zq.of_string) data in
+        if K.step1_check context cert then (
+          t.stt_cert <- Some cert;
+          t.stt_step <- Some 2)
+        else failwith "Invalid certificate"
+    | Some 3 ->
+        let certs = get_certs () in
+        let polynomial = polynomial_of_string (sread G.Zq.of_string) data in
+        if K.step3_check certs i polynomial then (
+          t.stt_polynomial <- Some polynomial;
+          t.stt_step <- Some 4)
+        else failwith "Invalid polynomial"
+    | Some 5 ->
+        let certs = get_certs () in
+        let polynomials = get_polynomials () in
+        let voutput =
+          voutput_of_string (sread G.of_string) (sread G.Zq.of_string) data
+        in
+        if K.step5_check certs i polynomials voutput then (
+          t.stt_voutput <-
+            Some
+              (string_of_voutput (swrite G.to_string) (swrite G.Zq.to_string)
+                 voutput);
+          t.stt_step <- Some 6)
+        else failwith "Invalid voutput"
+    | _ -> failwith "Invalid step"
+  in
+  let () =
+    if Array.for_all (fun x -> x.stt_step = Some 2) ts then
+      try
+        let threshold = K.step2 (get_certs ()) in
+        assert (dtp.dtp_threshold = Some threshold);
+        Array.iter (fun x -> x.stt_step <- Some 3) ts
+      with e -> dtp.dtp_error <- Some (Printexc.to_string e)
+  in
+  let () =
+    if Array.for_all (fun x -> x.stt_step = Some 4) ts then
+      try
+        let certs = get_certs () in
+        let polynomials = get_polynomials () in
+        let vinputs = K.step4 certs polynomials in
+        for j = 0 to Array.length ts - 1 do
+          ts.(j).stt_vinput <- Some vinputs.(j)
+        done;
+        Array.iter (fun x -> x.stt_step <- Some 5) ts
+      with e -> dtp.dtp_error <- Some (Printexc.to_string e)
+  in
+  let () =
+    if Array.for_all (fun x -> x.stt_step = Some 6) ts then
+      try
+        let certs = get_certs () in
+        let polynomials = get_polynomials () in
+        let voutputs =
+          Array.map
+            (fun x ->
+              match x.stt_voutput with
+              | None -> failwith "Missing voutput"
+              | Some y ->
+                  voutput_of_string (sread G.of_string) (sread G.Zq.of_string) y)
+            ts
+        in
+        let p = K.step6 certs polynomials voutputs in
+        dtp.dtp_parameters <-
+          Some
+            (string_of_threshold_parameters (swrite G.to_string)
+               (swrite G.Zq.to_string) p);
+        Array.iter (fun x -> x.stt_step <- Some 7) ts
+      with e -> dtp.dtp_error <- Some (Printexc.to_string e)
+  in
+  se.se_trustees <-
+    se_trustees
+    |> string_of_draft_trustees (swrite G.Zq.to_string)
+    |> draft_trustees_of_string Yojson.Safe.read_json;
+  set fse
+
+let dispatch_credentials ~token endpoint method_ body s uuid (se, set) =
+  let module S = (val s : Storage.BACKEND) in
   match endpoint with
   | [ "token" ] -> (
       let@ _ = with_administrator token se in
@@ -924,14 +1124,13 @@ let dispatch_credentials ~token endpoint method_ body uuid se =
       let@ _ = with_administrator token se in
       match method_ with
       | `GET ->
-          handle_get_option (fun () ->
-              Web_persist.get_draft_private_credentials uuid)
+          handle_get_option (fun () -> S.get (Election (uuid, Private_creds)))
       | _ -> method_not_allowed)
   | [ "public" ] -> (
       match method_ with
       | `GET ->
           handle_get_option (fun () ->
-              Web_persist.get_draft_public_credentials uuid)
+              Web_persist.get_draft_public_credentials s uuid)
       | `POST -> (
           let@ who = with_administrator_or_credential_authority token se in
           if Web_persist.get_credentials_status uuid se <> `None then forbidden
@@ -947,13 +1146,15 @@ let dispatch_credentials ~token endpoint method_ body uuid se =
                     Lwt.fail @@ exn_of_generate_credentials_on_server_error e)
             | `CredentialAuthority, credentials ->
                 let@ () = handle_generic_error in
-                let* () = submit_public_credentials uuid se credentials in
+                let* () =
+                  submit_public_credentials s uuid (se, set) credentials
+                in
                 ok
             | _ -> forbidden)
       | _ -> method_not_allowed)
   | _ -> not_found
 
-let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
+let dispatch_draft ~token ~ifmatch endpoint method_ body s uuid (se, set) =
   match endpoint with
   | [] -> (
       let@ who = with_administrator_or_nobody token se in
@@ -967,30 +1168,33 @@ let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
           let@ () = handle_ifmatch ifmatch get in
           let@ draft = body.run draft_of_string in
           let@ () = handle_generic_error in
-          let update_cache =
-            let (Draft (_, se)) = se in
-            let (Belenios_api.Common.Draft (_, draft)) = draft in
-            draft.draft_questions.t_name <> se.se_questions.t_name
-            || se.se_owners <> draft.draft_owners
-          in
           let se = draft_of_api account uuid se draft in
-          let* () = Web_persist.set_draft_election uuid se in
-          let* () =
-            if update_cache then Web_persist.clear_elections_by_owner_cache ()
-            else Lwt.return_unit
-          in
+          let* () = set se in
           ok
       | `POST, `Administrator a ->
           let@ () = handle_ifmatch ifmatch get in
           let@ x = body.run draft_request_of_string in
           let@ () = handle_generic_error in
-          post_draft_status ~admin_id:a.id uuid se x
+          post_draft_status ~admin_id:a.id s uuid (se, set) x
       | `DELETE, `Administrator _ ->
           let@ () = handle_ifmatch ifmatch get in
           let@ () = handle_generic_error in
-          let* () = Web_persist.delete_draft uuid in
+          let* () = Web_persist.delete_draft s uuid in
           ok
       | _ -> method_not_allowed)
+  | [ "election" ] -> (
+      let get () =
+        let (Draft (v, se)) = se in
+        let version = se.se_version in
+        let group = se.se_group in
+        let module G = (val Group.of_string ~version group : GROUP) in
+        let public_key = G.to_string G.g in
+        Lwt.return
+        @@ Election.make_raw_election ~version
+             (Template (v, se.se_questions))
+             ~uuid ~group ~public_key
+      in
+      match method_ with `GET -> handle_get get | _ -> method_not_allowed)
   | [ "voters" ] -> (
       let@ who = with_administrator_or_credential_authority token se in
       let get () =
@@ -1005,7 +1209,7 @@ let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
           else
             let@ voters = body.run voter_list_of_string in
             let@ () = handle_generic_error in
-            let* () = put_draft_voters uuid se voters in
+            let* () = put_draft_voters (se, set) voters in
             ok
       | `POST, `Administrator account -> (
           let@ () = handle_ifmatch ifmatch get in
@@ -1013,8 +1217,8 @@ let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
           let@ () = handle_generic_error in
           match request with
           | `Import from -> (
-              let@ _ = check_owner account from in
-              let* x = import_voters uuid se from in
+              let@ _ = check_owner account s from in
+              let* x = import_voters s uuid (se, set) from in
               match x with
               | Ok () -> ok
               | Stdlib.Error `Forbidden -> forbidden
@@ -1045,47 +1249,80 @@ let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
               Mails_voter.generate_password_email metadata langs title uuid id
                 show_weight
           in
-          let* jobs = post_draft_passwords account generate uuid se voters in
+          let* jobs = post_draft_passwords account generate (se, set) voters in
           let* () = Mails_voter.submit_bulk_emails jobs in
           ok
       | _ -> method_not_allowed)
   | "credentials" :: endpoint ->
-      dispatch_credentials ~token endpoint method_ body uuid se
-  | [ "trustees-pedersen" ] -> (
-      let@ index, trustee, dtp = with_threshold_trustee token se in
+      dispatch_credentials ~token endpoint method_ body s uuid (se, set)
+  | [ "trustee" ] -> (
+      let@ trustee = with_trustee token se in
       let get () =
-        let pedersen_certs =
-          List.fold_left
-            (fun accu x ->
-              match x.stt_cert with None -> accu | Some c -> c :: accu)
-            [] dtp.dtp_trustees
-          |> List.rev |> Array.of_list
+        let@ () =
+         fun cont ->
+          Lwt.return
+          @@ string_of_trustee_status Yojson.Safe.write_json
+               Yojson.Safe.write_json
+          @@ cont ()
         in
-        let (Draft (_, draft)) = se in
-        let pedersen_context =
-          {
-            group = draft.se_group;
-            size = List.length dtp.dtp_trustees;
-            threshold = Option.value ~default:0 dtp.dtp_threshold;
-            index;
-          }
-        in
-        let r =
-          {
-            pedersen_context;
-            pedersen_step = Option.value ~default:0 trustee.stt_step;
-            pedersen_certs;
-            pedersen_vinput = trustee.stt_vinput;
-            pedersen_voutput =
-              Option.map
-                (voutput_of_string Yojson.Safe.read_json Yojson.Safe.read_json)
-                trustee.stt_voutput;
-          }
-        in
-        Lwt.return
-        @@ string_of_pedersen Yojson.Safe.write_json Yojson.Safe.write_json r
+        match trustee with
+        | `Basic b -> `Basic (if b.st_public_key = "" then `Init else `Done)
+        | `Threshold (index, t, dtp) -> (
+            let@ () = fun cont -> `Threshold (cont ()) in
+            match dtp.dtp_threshold with
+            | None -> `Init
+            | Some threshold -> (
+                let (Draft (_, draft)) = se in
+                let pedersen_context =
+                  {
+                    group = draft.se_group;
+                    size = List.length dtp.dtp_trustees;
+                    threshold;
+                    index;
+                  }
+                in
+                match t.stt_cert with
+                | None -> `WaitingForCertificate pedersen_context
+                | Some _ -> (
+                    try
+                      let pedersen_certs =
+                        List.map
+                          (fun x ->
+                            match x.stt_cert with
+                            | None -> raise Exit
+                            | Some c -> c)
+                          dtp.dtp_trustees
+                        |> Array.of_list
+                      in
+                      `Pedersen
+                        {
+                          pedersen_context;
+                          pedersen_step = Option.value ~default:0 t.stt_step;
+                          pedersen_certs;
+                          pedersen_vinput = t.stt_vinput;
+                          pedersen_voutput =
+                            Option.map
+                              (voutput_of_string Yojson.Safe.read_json
+                                 Yojson.Safe.read_json)
+                              t.stt_voutput;
+                        }
+                    with Exit -> `WaitingForOtherCertificates)))
       in
-      match method_ with `GET -> handle_get get | _ -> method_not_allowed)
+      match method_ with
+      | `GET -> handle_get get
+      | `POST -> (
+          let@ data = body.run Fun.id in
+          let@ () = handle_generic_error in
+          match trustee with
+          | `Basic b ->
+              let* () = post_trustee_basic (se, set) ~token:b.st_token data in
+              ok
+          | `Threshold (_, t, _) ->
+              let* () =
+                post_trustee_threshold (se, set) ~token:t.stt_token data
+              in
+              ok)
+      | _ -> method_not_allowed)
   | [ "trustees" ] -> (
       let@ who = with_administrator_or_nobody token se in
       let get is_admin () =
@@ -1104,17 +1341,17 @@ let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
           let@ () = handle_generic_error in
           match request with
           | `Add trustee ->
-              let* () = post_draft_trustees uuid se trustee in
+              let* () = post_draft_trustees (se, set) trustee in
               ok
           | `SetBasic ->
-              let* () = put_draft_trustees_mode uuid se `Basic in
+              let* () = put_draft_trustees_mode (se, set) `Basic in
               ok
           | `SetThreshold t ->
-              let* () = put_draft_trustees_mode uuid se (`Threshold t) in
+              let* () = put_draft_trustees_mode (se, set) (`Threshold t) in
               ok
           | `Import from -> (
-              let@ metadata = check_owner account from in
-              let* x = import_trustees uuid se from metadata in
+              let@ metadata = check_owner account s from in
+              let* x = import_trustees (se, set) s from metadata in
               match x with
               | Ok _ -> ok
               | Stdlib.Error e ->
@@ -1133,7 +1370,7 @@ let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
       match method_ with
       | `DELETE ->
           let@ () = handle_generic_error in
-          let* x = delete_draft_trustee uuid se trustee in
+          let* x = delete_draft_trustee (se, set) trustee in
           if x then ok else not_found
       | _ -> method_not_allowed)
   | [ "status" ] -> (
@@ -1141,27 +1378,22 @@ let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
       match method_ with
       | `GET ->
           let@ () = handle_generic_error in
-          let* x = get_draft_status uuid se in
-          Lwt.return (200, string_of_draft_status x)
+          let* x = get_draft_status s uuid se in
+          return_json 200 (string_of_draft_status x)
       | _ -> method_not_allowed)
   | _ -> not_found
 
-let dispatch ~token ~ifmatch endpoint method_ body =
+let dispatch s ~token ~ifmatch endpoint method_ body =
   match endpoint with
   | [] -> (
       let@ token = Option.unwrap unauthorized token in
       let@ account = Option.unwrap unauthorized (lookup_token token) in
       let get () =
-        let* elections = Web_persist.get_elections_by_owner account.id in
+        let* elections = Storage.get_elections_by_owner account.id in
         let elections =
           List.fold_left
-            (fun accu (state, summary_uuid, date, summary_name) ->
-              let summary_date = Datetime.to_unixfloat date in
-              let summary_state = None in
-              if state = `Draft then
-                { summary_uuid; summary_name; summary_date; summary_state }
-                :: accu
-              else accu)
+            (fun accu ({ state; _ } as x) ->
+              if state = `Draft then x :: accu else accu)
             [] elections
         in
         Lwt.return @@ string_of_summary_list elections
@@ -1172,13 +1404,13 @@ let dispatch ~token ~ifmatch endpoint method_ body =
           let@ () = handle_ifmatch ifmatch get in
           let@ draft = body.run draft_of_string in
           let@ () = handle_generic_error in
-          let* uuid = post_drafts account draft in
+          let* uuid = post_drafts account s draft in
           match uuid with
-          | Some uuid -> Lwt.return (200, string_of_uuid uuid)
+          | Some uuid -> return_json 200 (string_of_uuid uuid)
           | None -> forbidden)
       | _ -> method_not_allowed)
   | uuid :: endpoint ->
       let@ uuid = Option.unwrap bad_request (Option.wrap Uuid.wrap uuid) in
-      let* se = Web_persist.get_draft_election uuid in
+      let* se = Spool.update s uuid Spool.draft in
       let@ se = Option.unwrap not_found se in
-      dispatch_draft ~token ~ifmatch endpoint method_ body uuid se
+      dispatch_draft ~token ~ifmatch endpoint method_ body s uuid se
