@@ -21,7 +21,7 @@
 
 open Lwt.Syntax
 open Belenios
-open Web_serializable_j
+open Belenios_server_core
 open Belenios_api.Serializable_j
 open Web_common
 open Api_generic
@@ -32,8 +32,8 @@ let with_administrator token metadata f =
   | Some a when Accounts.check a metadata.e_owners -> f a
   | _ -> unauthorized
 
-let find_trustee_id uuid token =
-  let* x = Web_persist.get_decryption_tokens uuid in
+let find_trustee_id s uuid token =
+  let* x = Spool.get s uuid Spool.decryption_tokens in
   match x with
   | None -> Lwt.return (int_of_string_opt token)
   | Some tokens ->
@@ -43,11 +43,11 @@ let find_trustee_id uuid token =
       in
       Lwt.return (find 1 tokens)
 
-let find_trustee_private_key uuid trustee_id =
-  let* keys = Web_persist.get_private_keys uuid in
+let find_trustee_private_key s uuid trustee_id =
+  let* keys = Spool.get s uuid Spool.private_keys in
   let&* keys = keys in
   (* there is one Pedersen trustee *)
-  let* trustees = Web_persist.get_trustees uuid in
+  let* trustees = Public_archive.get_trustees s uuid in
   let trustees =
     trustees_of_string Yojson.Safe.read_json Yojson.Safe.read_json trustees
   in
@@ -59,66 +59,62 @@ let find_trustee_private_key uuid trustee_id =
   in
   loop (trustee_id - 1) trustees
 
-let with_tally_trustee token uuid f =
+let with_tally_trustee token s uuid f =
   let@ token = Option.unwrap unauthorized token in
-  let* x = find_trustee_id uuid token in
-  match x with
-  | Some trustee_id ->
-      let* tally_trustee_private_key =
-        find_trustee_private_key uuid trustee_id
-      in
-      f { tally_trustee_private_key }
-  | None -> unauthorized
+  let* x = find_trustee_id s uuid token in
+  match x with Some trustee_id -> f trustee_id | None -> unauthorized
 
-let get_election_status uuid =
-  let* status_state = Web_persist.get_election_state uuid in
-  let* d = Web_persist.get_election_dates uuid in
+let get_election_status s uuid =
+  let* status_state = Web_persist.get_election_state s uuid in
+  let* d = Web_persist.get_election_dates s uuid in
+  let* status_authentication =
+    let* m = Web_persist.get_election_metadata s uuid in
+    Lwt.return @@ Api_drafts.authentication_of_auth_config m.e_auth_config
+  in
   let status_auto_archive_date =
     match status_state with
     | `Tallied ->
-        let t = Option.value d.e_tally ~default:Web_defaults.tally_date in
+        let t = Option.value d.e_tally ~default:Defaults.tally_date in
         Some
           (Datetime.to_unixfloat
-          @@ Period.add t (Period.day Web_defaults.days_to_archive))
+          @@ Period.add t (Period.day Defaults.days_to_archive))
     | _ -> None
   in
   let status_auto_delete_date =
     match status_state with
     | `Open | `Closed | `Shuffling | `EncryptedTally ->
         let t =
-          Option.value d.e_finalization ~default:Web_defaults.validation_date
+          Option.value d.e_finalization ~default:Defaults.validation_date
         in
         Datetime.to_unixfloat
-        @@ Period.add t (Period.day Web_defaults.days_to_delete)
+        @@ Period.add t (Period.day Defaults.days_to_delete)
     | `Tallied ->
-        let t = Option.value d.e_tally ~default:Web_defaults.tally_date in
+        let t = Option.value d.e_tally ~default:Defaults.tally_date in
         Datetime.to_unixfloat
-        @@ Period.add t
-             (Period.day Web_defaults.(days_to_archive + days_to_delete))
+        @@ Period.add t (Period.day Defaults.(days_to_archive + days_to_delete))
     | `Archived ->
-        let t = Option.value d.e_archive ~default:Web_defaults.archive_date in
+        let t = Option.value d.e_archive ~default:Defaults.archive_date in
         Datetime.to_unixfloat
-        @@ Period.add t (Period.day Web_defaults.days_to_delete)
+        @@ Period.add t (Period.day Defaults.days_to_delete)
   in
-  let* postpone = Web_persist.get_election_result_hidden uuid in
   Lwt.return
     {
-      status_state;
+      status_state :> state;
+      status_authentication;
       status_auto_archive_date;
       status_auto_delete_date;
-      status_postpone_date = Option.map Datetime.to_unixfloat postpone;
     }
 
-let get_partial_decryptions uuid metadata =
+let get_partial_decryptions s uuid metadata =
   let@ () =
    fun cont ->
-    let* state = Web_persist.get_election_state uuid in
+    let* state = Web_persist.get_election_state s uuid in
     match state with
     | `EncryptedTally -> cont ()
     | _ -> Lwt.fail @@ Error `NotInExpectedState
   in
-  let* pds = Web_persist.get_partial_decryptions uuid in
-  let* trustees = Web_persist.get_trustees uuid in
+  let* pds = Public_archive.get_partial_decryptions s uuid in
+  let* trustees = Public_archive.get_trustees s uuid in
   let trustees =
     trustees_of_string Yojson.Safe.read_json Yojson.Safe.read_json trustees
   in
@@ -151,7 +147,7 @@ let get_partial_decryptions uuid metadata =
     match threshold with
     | None -> Lwt.return @@ List.map string_of_int (seq 1 (npks + 1))
     | Some _ -> (
-        let* x = Web_persist.get_decryption_tokens uuid in
+        let* x = Spool.get s uuid Spool.decryption_tokens in
         match x with
         | Some x -> Lwt.return x
         | None -> (
@@ -159,7 +155,7 @@ let get_partial_decryptions uuid metadata =
             | None -> failwith "missing trustees in get_tokens_decrypt"
             | Some ts ->
                 let ts = List.map (fun _ -> generate_token ()) ts in
-                let* () = Web_persist.set_decryption_tokens uuid ts in
+                let* () = Spool.create s uuid Spool.decryption_tokens ts in
                 Lwt.return ts))
   in
   Lwt.return
@@ -175,34 +171,19 @@ let get_partial_decryptions uuid metadata =
       partial_decryptions_threshold = threshold;
     }
 
-let set_postpone_date uuid date =
-  let@ date cont =
-    match date with
-    | None -> cont None
-    | Some t ->
-        let t = Datetime.from_unixfloat t in
-        let max =
-          Period.add (Datetime.now ())
-            (Period.day Web_defaults.days_to_publish_result)
-        in
-        if Datetime.compare t max > 0 then Lwt.return_false else cont (Some t)
-  in
-  let* () = Web_persist.set_election_result_hidden uuid date in
-  Lwt.return_true
-
-let get_shuffles uuid metadata =
+let get_shuffles s uuid metadata =
   let@ () =
    fun cont ->
-    let* state = Web_persist.get_election_state uuid in
+    let* state = Web_persist.get_election_state s uuid in
     match state with
     | `Shuffling -> cont ()
     | _ -> Lwt.fail @@ Error `NotInExpectedState
   in
-  let* shuffles = Web_persist.get_shuffles uuid in
+  let* shuffles = Public_archive.get_shuffles s uuid in
   let shuffles = Option.value shuffles ~default:[] in
-  let* skipped = Web_persist.get_skipped_shufflers uuid in
+  let* skipped = Spool.get s uuid Spool.skipped_shufflers in
   let skipped = Option.value skipped ~default:[] in
-  let* token = Web_persist.get_shuffle_token uuid in
+  let* token = Web_persist.get_shuffle_token s uuid in
   Lwt.return
     {
       shuffles_shufflers =
@@ -235,39 +216,106 @@ let extract_names trustees =
   |> List.flatten
   |> List.mapi (fun i x -> (i + 1, x))
 
-let get_trustee_names uuid =
-  let* trustees = Web_persist.get_trustees uuid in
+let get_trustee_names s uuid =
+  let* trustees = Public_archive.get_trustees s uuid in
   let trustees =
     trustees_of_string Yojson.Safe.read_json Yojson.Safe.read_json trustees
   in
   Lwt.return (extract_names trustees)
 
-let get_trustee_name uuid metadata trustee =
+let get_trustee_name s uuid metadata trustee =
   match metadata.e_trustees with
   | None -> Lwt.return (1, None)
   | Some xs ->
-      let* names = get_trustee_names uuid in
+      let* names = get_trustee_names s uuid in
       Lwt.return (List.assoc trustee (List.combine xs names))
 
-let skip_shuffler uuid trustee =
-  let* x = Web_persist.get_shuffle_token uuid in
+let skip_shuffler s uuid trustee =
+  let* x = Web_persist.get_shuffle_token s uuid in
   let* () =
     match x with
-    | Some x when x.tk_trustee = trustee -> Web_persist.clear_shuffle_token uuid
+    | Some x when x.tk_trustee = trustee -> Spool.del s uuid Spool.shuffle_token
     | None -> Lwt.return_unit
     | _ -> Lwt.fail @@ Error `NotInExpectedState
   in
-  let* x = Web_persist.get_skipped_shufflers uuid in
-  let x = Option.value x ~default:[] in
-  if List.mem trustee x then Lwt.fail @@ Error `NotInExpectedState
-  else Web_persist.set_skipped_shufflers uuid (trustee :: x)
+  let@ current, set =
+   fun cont ->
+    let* x = Spool.update s uuid Spool.skipped_shufflers in
+    match x with
+    | None -> cont ([], Spool.create s uuid Spool.skipped_shufflers)
+    | Some x -> cont x
+  in
+  if List.mem trustee current then Lwt.fail @@ Error `NotInExpectedState
+  else set (trustee :: current)
 
-let select_shuffler uuid metadata trustee =
-  let* trustee_id, name = get_trustee_name uuid metadata trustee in
-  let* () = Web_persist.clear_shuffle_token uuid in
-  let* _ = Web_persist.gen_shuffle_token uuid trustee trustee_id name in
+let select_shuffler s uuid metadata trustee =
+  let* trustee_id, name = get_trustee_name s uuid metadata trustee in
+  let* () = Spool.del s uuid Spool.shuffle_token in
+  let* _ = Web_persist.gen_shuffle_token s uuid trustee trustee_id name in
   Lwt.return_unit
 
+let post_partial_decryption s uuid election ~trustee_id ~partial_decryption =
+  let* pds = Public_archive.get_partial_decryptions s uuid in
+  let@ () =
+   fun cont ->
+    if List.exists (fun x -> x.owned_owner = trustee_id) pds then
+      Lwt.return @@ Stdlib.Error `AlreadyDone
+    else cont ()
+  in
+  let module W = (val election : Election.ELECTION) in
+  let* pks =
+    let* trustees = Public_archive.get_trustees s uuid in
+    trustees
+    |> trustees_of_string W.(sread G.of_string) W.(sread G.Zq.of_string)
+    |> List.map (function
+         | `Single x -> [ x ]
+         | `Pedersen t -> Array.to_list t.t_verification_keys)
+    |> List.flatten |> Array.of_list |> Lwt.return
+  in
+  let pk = pks.(trustee_id - 1).trustee_public_key in
+  let pd =
+    partial_decryption_of_string
+      W.(sread G.of_string)
+      W.(sread G.Zq.of_string)
+      partial_decryption
+  in
+  let* et =
+    let* x = Public_archive.get_latest_encrypted_tally s uuid in
+    match x with
+    | None -> assert false
+    | Some x -> Lwt.return @@ encrypted_tally_of_string W.(sread G.of_string) x
+  in
+  if
+    string_of_partial_decryption
+      W.(swrite G.to_string)
+      W.(swrite G.Zq.to_string)
+      pd
+    = partial_decryption
+    && W.E.check_factor et pk pd
+  then
+    let pd = (trustee_id, partial_decryption) in
+    let* () = Web_persist.add_partial_decryption s uuid pd in
+    Lwt.return @@ Ok ()
+  else Lwt.return @@ Stdlib.Error `Invalid
+
+let post_shuffle s uuid election ~token ~shuffle =
+  let* expected_token = Web_persist.get_shuffle_token s uuid in
+  match expected_token with
+  | Some x when token = x.tk_token ->
+      Lwt.catch
+        (fun () ->
+          let* y =
+            Web_persist.append_to_shuffles s election x.tk_trustee_id shuffle
+          in
+          match y with
+          | Some _ ->
+              let* () = Spool.del s uuid Spool.shuffle_token in
+              let* () = Spool.del s uuid Spool.audit_cache in
+              Lwt.return @@ Ok ()
+          | None -> Lwt.return @@ Stdlib.Error `Failure)
+        (fun e -> Lwt.return @@ Stdlib.Error (`Invalid e))
+  | _ -> Lwt.return @@ Stdlib.Error `Forbidden
+
 let split_voting_record =
   let rex = Re.Pcre.regexp "\"(.*)(\\..*)?\" \".*:(.*)\"" in
   fun x ->
@@ -278,49 +326,62 @@ let split_voting_record =
       vr_username = Re.Pcre.get_substring s 3;
     }
 
-let get_records uuid =
-  let* x = Web_persist.get_records uuid in
+let get_records s uuid =
+  let* x = Web_persist.get_records s uuid in
   let x = Option.value x ~default:[] in
   Lwt.return @@ List.map split_voting_record x
 
-let cast_ballot send_confirmation election ~rawballot ~user ~precast_data =
-  let module W = (val election : Site_common_sig.ELECTION) in
-  let uuid = W.uuid in
+let cast_ballot send_confirmation s uuid election ~ballot ~user ~precast_data =
+  let module W = (val election : Election.ELECTION) in
   let* email, login, weight =
-    let* x = Web_persist.get_voter uuid user.user_name in
+    let* x = Web_persist.get_voter s uuid user.user_name in
     match x with
     | Some x -> Lwt.return @@ Voter.get x
-    | None -> fail UnauthorizedVoter
+    | None -> fail `UnauthorizedVoter
   in
-  let* show_weight = Web_persist.get_has_explicit_weights uuid in
+  let* show_weight = Web_persist.get_has_explicit_weights s uuid in
   let oweight = if show_weight then Some weight else None in
   let user_s = string_of_user user in
-  let* state = Web_persist.get_election_state uuid in
+  let* state = Web_persist.get_election_state s uuid in
   let voting_open = state = `Open in
-  let* () = if not voting_open then fail ElectionClosed else Lwt.return_unit in
+  let* () = if not voting_open then fail `ElectionClosed else Lwt.return_unit in
   let* r =
-    Web_persist.cast_ballot election ~rawballot ~user:user_s ~weight
+    Web_persist.cast_ballot s uuid ~ballot ~user:user_s ~weight
       (Datetime.now ()) ~precast_data
   in
   match r with
   | Ok (hash, revote) ->
-      let* success = send_confirmation uuid revote login email oweight hash in
+      let confirmation : confirmation =
+        {
+          user = login;
+          recipient = email;
+          hash;
+          revote;
+          weight = oweight;
+          email = false;
+        }
+      in
+      let* email = send_confirmation s uuid confirmation in
       let () =
         if revote then
           Printf.ksprintf Ocsigen_messages.accesslog
             "Someone revoted in election %s" (Uuid.unwrap uuid)
       in
-      Lwt.return (user, hash, revote, weight, success)
-  | Error e -> fail (CastError e)
+      Lwt.return { confirmation with email }
+  | Error e -> fail e
 
-let direct_voter_auth = ref (fun _ _ -> assert false)
+let direct_voter_auth = ref (fun _ _ _ -> assert false)
 (* initialized in Web_main *)
 
-let dispatch_election ~token ~ifmatch endpoint method_ body uuid raw metadata =
+let state_module = ref None
+(* initialized in Web_main *)
+
+let dispatch_election ~token ~ifmatch endpoint method_ body s uuid raw metadata
+    =
   match endpoint with
   | [] -> (
       let get () =
-        let* x = get_election_status uuid in
+        let* x = get_election_status s uuid in
         Lwt.return @@ string_of_election_status x
       in
       match method_ with
@@ -336,7 +397,7 @@ let dispatch_election ~token ~ifmatch endpoint method_ body uuid raw metadata =
                 | `Open -> Web_persist.open_election
                 | `Close -> Web_persist.close_election
               in
-              let* b = doit uuid in
+              let* b = doit s uuid in
               if b then ok else forbidden
           | (`ComputeEncryptedTally | `FinishShuffling) as x ->
               let@ () = handle_generic_error in
@@ -345,50 +406,41 @@ let dispatch_election ~token ~ifmatch endpoint method_ body uuid raw metadata =
                 | `ComputeEncryptedTally -> Web_persist.compute_encrypted_tally
                 | `FinishShuffling -> Web_persist.finish_shuffling
               in
-              let module W =
-                Belenios.Election.Make
-                  (struct
-                    let raw_election = raw
-                  end)
-                  (Random)
-                  ()
-              in
-              let* b = doit (module W) in
+              let* b = doit s uuid in
               if b then ok else forbidden
           | `ReleaseTally ->
               let@ () = handle_generic_error in
-              let* () = Web_persist.release_tally uuid in
+              let* () = Web_persist.release_tally s uuid in
               ok
           | `Archive ->
               let@ () = handle_generic_error in
-              let* () = Web_persist.archive_election uuid in
+              let* () = Web_persist.archive_election s uuid in
               ok
           | `RegeneratePassword user ->
               let@ () = handle_generic_error in
-              let module W =
-                Belenios.Election.Make
-                  (struct
-                    let raw_election = raw
-                  end)
-                  (Random)
-                  ()
-              in
-              let* b = Web_persist.regen_password (module W) metadata user in
-              if b then ok else not_found
-          | `SetPostponeDate date ->
-              let@ () = handle_generic_error in
-              let* b = set_postpone_date uuid date in
-              if b then ok else bad_request)
+              let* b = Web_persist.regen_password s uuid metadata user in
+              if b then ok else not_found)
       | `DELETE ->
           let@ () = handle_ifmatch ifmatch get in
           let@ _ = with_administrator token metadata in
           let@ () = handle_generic_error in
-          let* () = Web_persist.delete_election uuid in
+          let* () = Web_persist.delete_election s uuid in
           ok
       | _ -> method_not_allowed)
+  | [ "audit-cache" ] -> (
+      let get () =
+        let* x = Web_persist.get_audit_cache s uuid in
+        Lwt.return @@ string_of_audit_cache x
+      in
+      match method_ with `GET -> handle_get get | _ -> method_not_allowed)
   | [ "election" ] -> (
+      match method_ with `GET -> return_json 200 raw | _ -> method_not_allowed)
+  | [ "archive" ] -> (
       match method_ with
-      | `GET -> Lwt.return (200, raw)
+      | `GET ->
+          let module S = (val s) in
+          let* path = S.get_as_file (Election (uuid, Public_archive)) in
+          Lwt.return @@ `Bel path
       | _ -> method_not_allowed)
   | [ "salts"; index ] -> (
       match int_of_string_opt index with
@@ -396,18 +448,18 @@ let dispatch_election ~token ~ifmatch endpoint method_ body uuid raw metadata =
       | Some index -> (
           match method_ with
           | `GET -> (
-              let* x = Web_persist.get_salt uuid index in
+              let* x = Web_persist.get_salt s uuid index in
               match x with
               | None -> not_found
               | Some x ->
-                  Lwt.return (200, string_of_salt Yojson.Safe.write_json x))
+                  return_json 200 (string_of_salt Yojson.Safe.write_json x))
           | _ -> method_not_allowed))
   | [ "trustees" ] -> (
-      let get () = Web_persist.get_trustees uuid in
+      let get () = Public_archive.get_trustees s uuid in
       match method_ with `GET -> handle_get get | _ -> method_not_allowed)
   | [ "automatic-dates" ] -> (
       let get () =
-        let* x = Web_persist.get_election_automatic_dates uuid in
+        let* x = Web_persist.get_election_automatic_dates s uuid in
         Lwt.return @@ string_of_election_auto_dates x
       in
       match method_ with
@@ -417,47 +469,103 @@ let dispatch_election ~token ~ifmatch endpoint method_ body uuid raw metadata =
           let@ () = handle_ifmatch ifmatch get in
           let@ d = body.run election_auto_dates_of_string in
           let@ () = handle_generic_error in
-          let* () = Web_persist.set_election_automatic_dates uuid d in
+          let* () = Web_persist.set_election_automatic_dates s uuid d in
           ok
       | _ -> method_not_allowed)
   | [ "voters" ] -> (
       let@ _ = with_administrator token metadata in
       match method_ with
-      | `GET -> (
+      | `GET ->
           let@ () = handle_generic_error in
-          let* x = Web_persist.get_voters_file uuid in
-          match x with None -> not_found | Some x -> Lwt.return (200, x))
+          let module S = (val s) in
+          let* x = Web_persist.get_all_voters s uuid in
+          return_json 200 (string_of_voter_list x)
       | _ -> method_not_allowed)
   | [ "records" ] -> (
       let@ _ = with_administrator token metadata in
       match method_ with
       | `GET ->
           let@ () = handle_generic_error in
-          let* x = get_records uuid in
-          Lwt.return (200, string_of_records x)
+          let* x = get_records s uuid in
+          return_json 200 (string_of_records x)
       | _ -> method_not_allowed)
   | [ "partial-decryptions" ] -> (
       match method_ with
       | `GET ->
           let@ _ = with_administrator token metadata in
           let@ () = handle_generic_error in
-          let* x = get_partial_decryptions uuid metadata in
-          Lwt.return (200, string_of_partial_decryptions x)
+          let* x = get_partial_decryptions s uuid metadata in
+          return_json 200 (string_of_partial_decryptions x)
       | _ -> method_not_allowed)
-  | [ "tally-trustee" ] -> (
+  | [ "trustee" ] -> (
+      let* state = Web_persist.get_election_state s uuid in
+      match state with
+      | `EncryptedTally -> (
+          match method_ with
+          | `GET ->
+              let@ trustee_id = with_tally_trustee token s uuid in
+              let@ () = handle_generic_error in
+              let* tally_trustee_private_key =
+                find_trustee_private_key s uuid trustee_id
+              in
+              return_json 200
+                (string_of_tally_trustee { tally_trustee_private_key })
+          | `POST -> (
+              let@ trustee_id = with_tally_trustee token s uuid in
+              let@ () = handle_generic_error in
+              let@ partial_decryption = body.run Fun.id in
+              let module W = (val Election.of_string (module Random) raw) in
+              let* x =
+                post_partial_decryption s uuid
+                  (module W)
+                  ~trustee_id ~partial_decryption
+              in
+              match x with
+              | Ok () -> ok
+              | Error `AlreadyDone -> conflict
+              | Error `Invalid -> bad_request)
+          | _ -> method_not_allowed)
+      | `Shuffling -> (
+          match method_ with
+          | `POST -> (
+              let@ token = Option.unwrap unauthorized token in
+              let@ () = handle_generic_error in
+              let@ shuffle = body.run Fun.id in
+              let election = Election.of_string (module Random) raw in
+              let* x = post_shuffle s uuid election ~token ~shuffle in
+              match x with
+              | Ok () -> ok
+              | Error `Forbidden -> forbidden
+              | Error _ -> bad_request)
+          | _ -> method_not_allowed)
+      | _ -> precondition_failed)
+  | [ "nh-ciphertexts" ] -> (
       match method_ with
       | `GET ->
-          let@ x = with_tally_trustee token uuid in
           let@ () = handle_generic_error in
-          Lwt.return (200, string_of_tally_trustee x)
+          Lwt.catch
+            (fun () ->
+              let* x = Public_archive.get_nh_ciphertexts s uuid in
+              return_json 200 x)
+            (function Election_not_found _ -> not_found | e -> Lwt.reraise e)
+      | _ -> method_not_allowed)
+  | [ "encrypted-tally" ] -> (
+      match method_ with
+      | `GET ->
+          let@ () = handle_generic_error in
+          Lwt.catch
+            (fun () ->
+              let* x = Public_archive.get_latest_encrypted_tally s uuid in
+              match x with None -> not_found | Some x -> return_json 200 x)
+            (function Election_not_found _ -> not_found | e -> Lwt.reraise e)
       | _ -> method_not_allowed)
   | [ "shuffles" ] -> (
       match method_ with
       | `GET ->
           let@ _ = with_administrator token metadata in
           let@ () = handle_generic_error in
-          let* x = get_shuffles uuid metadata in
-          Lwt.return (200, string_of_shuffles x)
+          let* x = get_shuffles s uuid metadata in
+          return_json 200 (string_of_shuffles x)
       | _ -> method_not_allowed)
   | [ "shuffles"; shuffler ] -> (
       match method_ with
@@ -467,78 +575,116 @@ let dispatch_election ~token ~ifmatch endpoint method_ body uuid raw metadata =
           let@ () = handle_generic_error in
           match request with
           | `Skip ->
-              let* () = skip_shuffler uuid shuffler in
+              let* () = skip_shuffler s uuid shuffler in
               ok
           | `Select ->
-              let* () = select_shuffler uuid metadata shuffler in
+              let* () = select_shuffler s uuid metadata shuffler in
               ok)
       | _ -> method_not_allowed)
   | [ "ballots" ] -> (
       match method_ with
-      | `POST -> (
-          let@ token = Option.unwrap unauthorized token in
-          let@ user cont =
-            Lwt.catch
-              (fun () ->
-                let json =
-                  match Base64.decode token with
-                  | Ok x -> Yojson.Safe.from_string x
-                  | Error (`Msg x) -> failwith x
-                in
-                let* x = !direct_voter_auth uuid json in
-                cont x)
-              (fun _ -> unauthorized)
-          in
-          let@ rawballot = body.run Fun.id in
+      | `GET ->
           let@ () = handle_generic_error in
-          let send_confirmation _ _ _ _ _ _ = Lwt.return_true in
-          let module W =
-            Belenios.Election.Make
-              (struct
-                let raw_election = raw
-              end)
-              (Random)
-              ()
+          let* x = Public_archive.get_ballot_hashes s uuid in
+          return_json 200 (string_of_ballots_with_weights x)
+      | `POST -> (
+          let@ () = handle_generic_error in
+          let@ state_module cont =
+            match !state_module with
+            | None -> failwith "anomaly: state_module not initialized"
+            | Some x -> cont x
           in
-          let* x = Web_persist.precast_ballot (module W) ~rawballot in
-          match x with
-          | Error _ -> bad_request
-          | Ok precast_data ->
+          let module State = (val state_module : Web_auth_sig.STATE) in
+          let@ ballot = body.run Fun.id in
+          let@ precast_data cont =
+            Lwt.try_bind
+              (fun () -> Web_persist.precast_ballot s uuid ~ballot)
+              (function
+                | Ok x -> cont x | Error e -> raise @@ Error (`CastError e))
+              (function
+                | Election_not_found _ -> not_found | e -> Lwt.reraise e)
+          in
+          match token with
+          | None ->
+              let* state =
+                State.create s uuid { ballot; precast_data; api_request = true }
+              in
+              let json =
+                match state with
+                | None -> `Assoc []
+                | Some state -> `Assoc [ ("state", `String state) ]
+              in
+              return_json 401 (Yojson.Safe.to_string json)
+          | Some token ->
+              let@ user cont =
+                Lwt.catch
+                  (fun () ->
+                    let json =
+                      match Base64.decode token with
+                      | Ok x -> Yojson.Safe.from_string x
+                      | Error (`Msg x) -> failwith x
+                    in
+                    let* x = !direct_voter_auth s uuid json in
+                    cont x)
+                  (fun _ -> unauthorized)
+              in
+              let send_confirmation _ _ _ = Lwt.return_false in
               let* _ =
-                cast_ballot send_confirmation
-                  (module W)
-                  ~rawballot ~user ~precast_data
+                let election = Election.of_string (module Random) raw in
+                cast_ballot send_confirmation s uuid election ~ballot ~user
+                  ~precast_data
               in
               ok)
       | _ -> method_not_allowed)
+  | [ "objects"; hash ] -> (
+      match method_ with
+      | `GET -> (
+          let@ () = handle_generic_error in
+          let* x = Public_archive.get_data s uuid (Hash.of_hex hash) in
+          match x with Some x -> return_json 200 x | None -> not_found)
+      | _ -> method_not_allowed)
+  | [ "last-event" ] -> (
+      match method_ with
+      | `GET -> (
+          let@ () = handle_generic_error in
+          let* x = Spool.get s uuid Spool.last_event in
+          match x with
+          | Some x -> return_json 200 (string_of_last_event x)
+          | None -> not_found)
+      | _ -> method_not_allowed)
+  | [ "roots" ] -> (
+      match method_ with
+      | `GET ->
+          let@ () = handle_generic_error in
+          let* x = Public_archive.get_roots s uuid in
+          return_json 200 (string_of_roots x)
+      | _ -> method_not_allowed)
   | _ -> not_found
 
-let dispatch ~token ~ifmatch endpoint method_ body =
+let dispatch s ~token ~ifmatch endpoint method_ body =
   match endpoint with
   | [] -> (
       let@ token = Option.unwrap unauthorized token in
       let@ account = Option.unwrap unauthorized (lookup_token token) in
       match method_ with
       | `GET ->
-          let* elections = Web_persist.get_elections_by_owner account.id in
+          let* elections = Storage.get_elections_by_owner account.id in
           let elections =
             List.fold_left
-              (fun accu (state, summary_uuid, date, summary_name) ->
-                let summary_date = Datetime.to_unixfloat date in
+              (fun accu ({ state; _ } as x) ->
                 match state with
                 | `Draft -> accu
-                | ( `Open | `Closed | `Shuffling | `EncryptedTally | `Tallied
-                  | `Archived ) as x ->
-                    let summary_state = Some x in
-                    { summary_uuid; summary_name; summary_date; summary_state }
-                    :: accu)
+                | `Open | `Closed | `Shuffling | `EncryptedTally | `Tallied
+                | `Archived ->
+                    x :: accu)
               [] elections
           in
-          Lwt.return (200, string_of_summary_list elections)
+          return_json 200 (string_of_summary_list elections)
       | _ -> method_not_allowed)
   | uuid :: endpoint ->
       let@ uuid = Option.unwrap bad_request (Option.wrap Uuid.wrap uuid) in
-      let* raw = Web_persist.get_raw_election uuid in
+      let* raw = Public_archive.get_election s uuid in
       let@ raw = Option.unwrap not_found raw in
-      let* metadata = Web_persist.get_election_metadata uuid in
-      dispatch_election ~token ~ifmatch endpoint method_ body uuid raw metadata
+      let* metadata = Web_persist.get_election_metadata s uuid in
+      dispatch_election ~token ~ifmatch endpoint method_ body s uuid raw
+        metadata
