@@ -22,6 +22,7 @@
 open Lwt
 open Lwt.Syntax
 open Belenios
+open Belenios_server_core
 open Web_common
 open Eliom_content.Html.F
 open Eliom_content.Html.F.Form
@@ -42,9 +43,10 @@ struct
     let href = Xml.uri_of_string (Eliom_uri.make_string_uri ~service x) in
     Eliom_content.Html.F.Raw.a ~a:(a_href href :: a) contents
 
-  let absolute_uri_of_service ~service x =
-    Eliom_uri.make_string_uri ~absolute:true ~service x
-    |> rewrite_prefix |> Eliom_content.Xml.uri_of_string
+  let api_a endpoint uuid contents =
+    let path = (endpoint uuid).Belenios_api.Endpoints.path in
+    let href = Printf.sprintf "%s/api/%s" !Web_config.prefix path in
+    Eliom_content.Html.F.Raw.a ~a:[ a_href @@ Xml.uri_of_string href ] contents
 
   let static x =
     let service =
@@ -52,9 +54,8 @@ struct
         ~get_params:(Eliom_parameter.string "version")
         ()
     in
-    Eliom_uri.make_string_uri ~absolute:true ~service
-      ([ "static"; x ], Version.build)
-    |> rewrite_prefix |> Eliom_content.Xml.uri_of_string
+    make_absolute_string_uri ~service ([ "static"; x ], Version.build)
+    |> Eliom_content.Xml.uri_of_string
 
   let get_preferred_gettext () = Web_i18n.get_preferred_gettext "voter"
 
@@ -62,44 +63,38 @@ struct
     match file with
     | None -> return default
     | Some f -> (
-        let* f =
-          let f' = f ^ "." ^ lang in
-          let* b = Lwt_unix.file_exists f' in
-          return (if b then f' else f)
-        in
-        let* file = Filesystem.read_file f in
+        let* file = Filesystem.read_file_i18n ~lang f in
         match file with
         | None -> return default
-        | Some x -> return @@ Unsafe.data (String.concat "\n" x))
+        | Some x -> return @@ Unsafe.data x)
 
   module UiBase = struct
     module Xml = Eliom_content.Xml
     module Svg = Eliom_content.Svg.F.Raw'
     module Html = Eliom_content.Html.F.Raw'
 
-    module Uris = struct
-      let home = absolute_uri_of_service ~service:home ()
-      let logo = absolute_uri_of_service ~service:logo ()
-      let belenios = Eliom_content.Xml.uri_of_string Belenios_ui.Links.belenios
-      let source_code = absolute_uri_of_service ~service:source_code ()
-      let privacy_policy = Xml.uri_of_string !Web_config.gdpr_uri
-    end
+    let uris = Api_generic.get_configuration_uris ()
   end
 
   module Ui = Belenios_ui.Pages_common.Make (UiBase)
 
+  let markup = Ui.markup
+  let confirmation_fragment = Ui.confirmation_fragment
+
   let base ~title ?full_title ?(login_box = txt "") ?lang_box ~content
-      ?(footer = txt "") ?uuid ?static:(static_page = false) ?redirect () =
+      ?(footer = txt "") ?sticky_footer ?uuid ?static:(static_page = false)
+      ?redirect () =
     let* l = get_preferred_gettext () in
     let open (val l) in
     let administer =
       match uuid with
-      | None -> raw_a ~service:admin [ txt (s_ "Administer elections") ] ()
+      | None -> raw_a ~service:home [ txt (s_ "Administer elections") ] ()
       | Some uuid ->
-          raw_a ~service:election_admin
-            ~a:[ a_id ("election_admin_" ^ Uuid.unwrap uuid) ]
+          let uri = Xml.uri_of_string @@ make_admin_link (Some uuid) in
+          let id = Printf.sprintf "election_admin_%s" (Uuid.unwrap uuid) in
+          Eliom_content.Html.F.Raw.a
+            ~a:[ a_id id; a_href uri ]
             [ txt (s_ "Administer this election") ]
-            uuid
     in
     let lang_box =
       match lang_box with
@@ -144,7 +139,8 @@ struct
          (head (Eliom_content.Html.F.title (txt title)) head_content)
          (body
             (Ui.base_body l ~full_title ~login_box ~warning ~lang_box ~content
-               ~footer ~administer ~extra_footer ~restricted_mode ())))
+               ~footer ~administer ~extra_footer ?sticky_footer ~restricted_mode
+               ())))
 
   let lang_box cont =
     let cont = default_admin cont in
@@ -209,13 +205,12 @@ struct
        see https://bugzilla.mozilla.org/show_bug.cgi?id=646552 *)
     direct_a ~target:"_blank" uri contents
 
-  let generic_page ~title ?service message () =
+  let generic_page ~title ?uri ?service message () =
     let* l = get_preferred_gettext () in
     let open (val l) in
     let proceed =
-      match service with
-      | None -> txt ""
-      | Some service ->
+      match (uri, service) with
+      | None, Some service ->
           div
             [
               a ~service
@@ -223,6 +218,14 @@ struct
                 [ txt (s_ "Proceed") ]
                 ();
             ]
+      | Some uri, None ->
+          div
+            [
+              Eliom_content.Html.F.Raw.a
+                ~a:[ a_id "generic_proceed_link"; a_href uri ]
+                [ txt (s_ "Proceed") ];
+            ]
+      | _ -> txt ""
     in
     let content = [ p [ txt message ]; proceed ] in
     base ~title ~content ()
@@ -248,7 +251,12 @@ struct
     let open (val l) in
     let auth_systems =
       auth_systems
-      |> List.map (fun name -> a ~service:(service name) [ txt name ] ())
+      |> List.map (fun name ->
+             a
+               ~a:[ a_id (Printf.sprintf "login_%s" name) ]
+               ~service:(service name)
+               [ txt name ]
+               ())
       |> List.join (txt ", ")
     in
     let content =
@@ -497,9 +505,9 @@ struct
     add_newline b;
     add_string b "-- ";
     add_newline b;
-    add_string b (s_ "Belenios Server");
+    add_string b !Web_config.server_name;
     let body = contents b in
-    let subject = s_ "Belenios authentication" in
+    let subject = !Web_config.vendor ^^^ s_ "Authentication" in
     Lwt.return (subject, body)
 
   let signup_captcha_img challenge =
@@ -578,4 +586,15 @@ struct
     in
     let title = s_ "Redirection" in
     base ~title ~content ~redirect:uri ()
+
+  let make_unsafe_script x = Printf.ksprintf Unsafe.data "<script>%s</script>" x
+
+  let html_js_exec x =
+    html
+      (head (title (txt "Belenios")) [])
+      (body
+         [
+           script ~a:[ a_src (static "belenios_jslib.js") ] (txt "");
+           make_unsafe_script x;
+         ])
 end
