@@ -22,13 +22,12 @@
 module type ELECTION = Belenios.Election.ELECTION
 
 module type S = sig
-  val find_election : Web_serializable_t.uuid -> (module ELECTION) option Lwt.t
   val election_not_found : unit -> Eliom_registration.Html.result Lwt.t
 
   val with_election :
-    Web_serializable_t.uuid ->
-    ((module ELECTION) -> Eliom_registration.Html.result Lwt.t) ->
-    Eliom_registration.Html.result Lwt.t
+    (((module ELECTION) -> Eliom_registration.Html.result Lwt.t) ->
+    Eliom_registration.Html.result Lwt.t)
+    Belenios_server_core.Storage.u
 
   val redir_preapply :
     ( 'a,
