@@ -23,9 +23,11 @@ open Lwt
 open Lwt.Syntax
 open Eliom_service
 open Belenios
+open Belenios_server_core
 open Web_common
 
-module Make (Web_auth : Web_auth_sig.S) = struct
+module Make (Web_services : Web_services_sig.S) (Web_auth : Web_auth_sig.S) =
+struct
   let next_lf str i = String.index_from_opt str i '\n'
 
   let login_cas =
@@ -37,10 +39,9 @@ module Make (Web_auth : Web_auth_sig.S) = struct
       ()
 
   let cas_self ~state =
-    Eliom_uri.make_string_uri ~absolute:true
+    Web_services.make_absolute_string_uri
       ~service:(preapply ~service:login_cas (state, None))
       ()
-    |> rewrite_prefix
 
   let extract tag xs =
     let rec loop = function
@@ -66,7 +67,6 @@ module Make (Web_auth : Web_auth_sig.S) = struct
       let mail =
         extract "cas:attributes" info >>= extract "cas:mail" >>= extract_pcdata
       in
-      let mail = match mail with Some x -> x | None -> "" in
       Some (user, mail)
     with _ -> None
 
@@ -95,7 +95,7 @@ module Make (Web_auth : Web_auth_sig.S) = struct
         | "yes" ->
             let x =
               let& j = next_lf info (i + 1) in
-              Some (String.sub info (i + 1) (j - i - 1), "")
+              Some (String.sub info (i + 1) (j - i - 1), None)
             in
             `Yes x
         | "no" -> `No
@@ -127,7 +127,7 @@ module Make (Web_auth : Web_auth_sig.S) = struct
   let handler _ a =
     let module X = struct
       let pre_login_handler _ ~state =
-        match List.assoc_opt "server" a.Web_serializable_t.auth_config with
+        match List.assoc_opt "server" a.auth_config with
         | Some server ->
             let cas_login =
               Eliom_service.extern ~prefix:server ~path:[ "login" ]
@@ -135,10 +135,7 @@ module Make (Web_auth : Web_auth_sig.S) = struct
                 ()
             in
             let service = preapply ~service:cas_login (cas_self ~state) in
-            let url =
-              Eliom_uri.make_string_uri ~service ~absolute:true ()
-              |> rewrite_prefix
-            in
+            let url = Web_services.make_absolute_string_uri ~service () in
             return (Web_auth_sig.Redirection url, Web_auth.No_data)
         | _ -> failwith "cas_login_handler invoked with bad config"
 
@@ -154,9 +151,7 @@ module Make (Web_auth : Web_auth_sig.S) = struct
       {
         Web_auth.post_login_handler =
           (fun ~data:_ _ a cont ->
-            match
-              (ticket, List.assoc_opt "server" a.Web_serializable_t.auth_config)
-            with
+            match (ticket, List.assoc_opt "server" a.auth_config) with
             | Some x, Some server -> (
                 let* r = get_cas_validation server ~state x in
                 match r with
