@@ -22,7 +22,7 @@
 open Lwt
 open Lwt.Syntax
 open Belenios
-open Web_serializable_t
+open Belenios_server_core
 open Web_common
 
 type login_env = {
@@ -41,7 +41,7 @@ struct
     type t = int
 
     let equal = ( = )
-    let hash x = x
+    let hash = Fun.id
   end
 
   module Captcha_throttle = Lwt_throttle.Make (HashedInt)
@@ -79,10 +79,7 @@ struct
               let env = { username_or_address; state; auth_instance } in
               let* () = Eliom_reference.set login_env (Some env) in
               let service = Web_services.email_election_login in
-              let url =
-                Eliom_uri.make_string_uri ~service ~absolute:true ()
-                |> rewrite_prefix
-              in
+              let url = Web_services.make_absolute_string_uri ~service () in
               return (Web_auth_sig.Redirection url, Web_auth.No_data)
             else
               let* fragment =
@@ -117,7 +114,8 @@ struct
       | None -> return ((if is_email name then Some name else None), `Site)
       | Some uuid ->
           let* address =
-            let* x = Web_persist.get_voter uuid name in
+            let@ s = Storage.with_transaction in
+            let* x = Web_persist.get_voter s uuid name in
             match x with
             | None -> Lwt.return_none
             | Some v ->
@@ -147,9 +145,15 @@ struct
             Pages_common.authentication_impossible ()
             >>= Eliom_registration.Html.send
         | Some { username_or_address; state; auth_instance } -> (
-            let* precast_data = Eliom_reference.get Web_state.precast_data in
+            let@ precast_data cont =
+              let x = Web_auth.State.get ~state in
+              match x with
+              | Some { state = Some { precast_data; _ }; _ } ->
+                  cont (Some precast_data)
+              | _ -> cont None
+            in
             match precast_data with
-            | Some (_, { cr_username = Some name; _ }) ->
+            | Some { credential_record = { cr_username = Some name; _ }; _ } ->
                 handle_email_post ~show_email_address:true ~state name true
             | _ ->
                 let* fragment =
@@ -184,7 +188,7 @@ struct
                       match Otp.check ~address ~code with
                       | Some () ->
                           let* () = Eliom_state.discard ~scope () in
-                          return_some (name, address)
+                          return_some (name, Some address)
                       | None -> return_none
                     in
                     cont ok);
