@@ -19,10 +19,16 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
-open Web_serializable_t
+open Belenios_server_core
 
 type data = ..
 
+type state = {
+  ballot : string;
+  precast_data : Web_persist.precast_data;
+  api_request : bool;
+}
+
 type result =
   | Html : Html_types.div Eliom_content.Html.elt -> result
   | Redirection : string -> result
@@ -39,6 +45,20 @@ type auth_system = {
   extern : bool;
 }
 
+type env = { uuid : uuid; state : state option; user : user option }
+
+module type STATE = sig
+  val create : Storage.t -> uuid -> state -> string option Lwt.t
+  val get : state:string -> env option
+  val del : state:string -> unit
+
+  val get_result :
+    state:string -> Belenios_api.Serializable_t.cast_result option
+
+  val set_result :
+    state:string -> Belenios_api.Serializable_t.cast_result -> unit
+end
+
 module type S = sig
   type data += No_data
 
@@ -48,7 +68,7 @@ module type S = sig
       data:data ->
       uuid option ->
       auth_config ->
-      ((string * string) option -> 'a Lwt.t) ->
+      ((string * string option) option -> 'a Lwt.t) ->
       'a Lwt.t;
   }
 
@@ -60,5 +80,7 @@ module type S = sig
     Eliom_registration.Html.result Lwt.t
 
   val get_site_login_handler : string -> result Lwt.t
-  val direct_voter_auth : uuid -> Yojson.Safe.t -> user Lwt.t
+  val direct_voter_auth : (Yojson.Safe.t -> user Lwt.t) Storage.u
+
+  module State : STATE
 end
