@@ -19,7 +19,7 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
-open Web_serializable_t
+open Belenios_server_core
 
 val prefix : string ref
 val site_auth_config : auth_config list ref
@@ -27,14 +27,14 @@ val site_auth_config : auth_config list ref
 val exported_auth_config :
   [ `BuiltinPassword | `BuiltinCAS | `Export of auth_config ] list ref
 
-val locales_dir : string ref
-val spool_dir : string ref
-val accounts_dir : string ref
+val share_dir : string ref
+val vendor : string ref
 val server_name : string ref
 val server_mail : string ref
 val return_path : string option ref
 val contact_uri : string option ref
-val gdpr_uri : string ref
+val tos : string ref
+val tos_last_update : float ref
 val warning_file : string option ref
 val footer_file : string option ref
 val admin_home : string option ref
@@ -44,7 +44,6 @@ val logo : (string * string) option ref
 val favicon : (string * string) option ref
 val sealing : (string * string) option ref
 val maxmailsatonce : int ref
-val uuid_length : int option ref
 val default_group : string ref
 val nh_group : string ref
 val domain : string ref
