@@ -19,29 +19,27 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
-open Lwt
-open Lwt.Syntax
-
 module Make () = struct
-  let default_scope = Eliom_common.default_session_scope
   let belenios_scope = `Session (Eliom_common.create_scope_hierarchy "belenios")
-  let show_cookie_disclaimer = Eliom_reference.eref ~scope:default_scope true
   let site_user = Eliom_reference.eref ~scope:belenios_scope None
-  let election_user = Eliom_reference.eref ~scope:belenios_scope None
-
-  let get_election_user uuid =
-    let* user = Eliom_reference.get election_user in
-    match user with
-    | Some (u, x) when u = uuid -> return_some x
-    | _ -> return_none
-
-  let ballot = Eliom_reference.eref ~scope:belenios_scope None
-  let precast_data = Eliom_reference.eref ~scope:belenios_scope None
-  let cast_confirmed = Eliom_reference.eref ~scope:belenios_scope None
-  let language = Eliom_reference.eref ~scope:default_scope None
   let signup_address = Eliom_reference.eref ~scope:belenios_scope None
   let signup_env = Eliom_reference.eref ~scope:belenios_scope None
   let set_email_env = Eliom_reference.eref ~scope:belenios_scope None
   let billing_env = Eliom_reference.eref ~scope:belenios_scope None
   let discard () = Eliom_state.discard ~scope:belenios_scope ()
+
+  let get_consent_cookie () =
+    let cookies = Eliom_request_info.get_cookies () in
+    match Ocsigen_cookie_map.Map_inner.find_opt "belenios-consent" cookies with
+    | None -> true
+    | Some x -> (
+        match float_of_string x with
+        | exception _ -> true
+        | x -> x < !Web_config.tos_last_update)
+
+  let set_consent_cookie () =
+    let now = Unix.gettimeofday () in
+    let exp = now +. (10. *. 365. *. 86400.) in
+    Eliom_state.set_cookie ~exp ~name:"belenios-consent"
+      ~value:(string_of_float now) ()
 end
