@@ -19,29 +19,19 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
-open Belenios
+open Belenios_server_core
 open Web_common
-open Web_serializable_t
 
 type signup_kind = CreateAccount | ChangePassword of { username : string }
 type signup_env = { kind : signup_kind; service : string }
 
 module type S = sig
-  val show_cookie_disclaimer : bool Eliom_reference.eref
   val site_user : (user * account * string) option Eliom_reference.eref
-  val election_user : (uuid * user) option Eliom_reference.eref
-  val get_election_user : uuid -> user option Lwt.t
-  val ballot : string option Eliom_reference.eref
-  val precast_data : (string * credential_record) option Eliom_reference.eref
-
-  val cast_confirmed :
-    (user * string * bool * Weight.t * bool, Web_common.error) result option
-    Eliom_reference.eref
-
-  val language : string option Eliom_reference.eref
   val signup_address : string option Eliom_reference.eref
   val signup_env : signup_env option Eliom_reference.eref
   val set_email_env : string option Eliom_reference.eref
   val billing_env : site_cont option Eliom_reference.eref
   val discard : unit -> unit Lwt.t
+  val get_consent_cookie : unit -> bool
+  val set_consent_cookie : unit -> unit
 end
